// Copyright (c) 2010 Robert V Mahoney Jr
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// hardware I2C interface

#define _EXTERN_ extern
#include "baplib.h"
#include "hi2c.h"

//#if (_HARDWARE_I2C_ == 1)

//#define _I2C_CLOCK_PAUSE_US_			20

#define _I2C_BUS_WAIT_PAUSE_			500

#define _I2C_MST_SLAVE_					0
#define _I2C_MST_MASTER_				1
#define _I2C_TRS_RECEIVE_				0
#define _I2C_TRS_TRANSMIT_				1
#define _I2C_MLS_MSB_FIRST_				0
#define _I2C_MLS_LSB_FIRST_				1
#define _I2C_WAIT_BIT_DISABLE_			0
#define _I2C_WAIT_BIT_ENABLE_			1
#define _I2C_SDAO_LOW_					0
#define _I2C_SDAO_HI_Z_					1
#define _I2C_BITS_9_					0
#define _I2C_ACK_JUDGEMENT_IGNORE_		0
#define _I2C_ACK_JUDGEMENT_HALT_		1

#define _I2C_SEND_ACK_ 		1		// I2C interface continue read sequence
#define _I2C_SEND_NACK_		0		// I2C interface terminate read sequence


void HI2CStart ();
void HI2CStop ();
void HI2CConnect (u8 _bDeviceAddr, u8 _bAddressType, u16 _wIOAddress);
void HI2CWriteByte (u8 _bData);
void HI2CReadByte (u8 *_pbData, u8 _bSendAck);

u8 __bHI2CDeviceAddr;
u8 __bHI2CBusSpeed = 0xFF;
u8 __bHI2CShowAckError = true;
u8 __bHI2CInit = false;
u8 g_bError;
void HI2CInitialize (u8 _bBusSpeed)
{
    __bHI2CDeviceAddr = 0x00;
	IIC2.ICCR1.BYTE = 0x00;
	IIC2.ICCR1.BIT.ICE = 1; // enable I2C and set I2C pins as I2C pins
//	IIC2.ICCR1.BIT.MST = _I2C_MST_SLAVE_; // default; select slave mode
	IIC2.ICCR1.BIT.MST = _I2C_MST_MASTER_; // select master mode
	IIC2.ICCR1.BIT.TRS = _I2C_TRS_RECEIVE_; // default; select receive mode
	IIC2.ICCR1.BIT.CKS = _bBusSpeed; // set initial clock
	IIC2.ICMR.BYTE = 0x00;
//	IIC2.ICMR.BIT.MLS = _I2C_MLS_MSB_FIRST_; // default; MSB first
//	IIC2.ICMR.BIT.BC = _I2C_BITS_9_; // default; 9 bits
//	IIC2.ICMR.BIT.WAIT = _I2C_WAIT_BIT_DISABLE_; // default; disable wait
//IIC2.ICMR.BIT.WAIT = _I2C_WAIT_BIT_ENABLE_; // enable wait;
//	IIC2.ICIER.BIT.ACKE = _I2C_ACK_JUDGEMENT_IGNORE_; // default; continue on ACK error
//	IIC2.ICCR2.BIT.SDAO = _I2C_SDAO_HI_Z_; // default; SDA is set for Hi-Z using external pullups
	__bHI2CInit = true;
}

void HI2CShowAckError (u8 _bShowAckError)
{
	__bHI2CShowAckError = _bShowAckError;
}

void HI2CTerminate (void)
{
//	IIC2.ICCR1.BIT.MST = _I2C_MST_SLAVE_; // select slave mode
	IIC2.ICCR1.BIT.TRS = _I2C_TRS_RECEIVE_; // select receive mode
	IIC2.ICCR1.BIT.ICE = 0; // disable I2C and set I2C pins as IO pins
}

void HI2CStart ()
{
	#if (_I2C_LOG_ == 1)
	DBGPrintf ("\nHI2C start");
	#endif
//	while (IIC2.ICCR2.BIT.BBSY != _CLEAR_) { // wait for bus to be free
//		DBGPrintf ("\nI2C bus busy");
//IIC2.ICCR2.BIT.BBSY = _CLEAR_;
//		Pause (_I2C_BUS_WAIT_PAUSE_);
//	}
	IIC2.ICCR2.BYTE = ((IIC2.ICCR2.BYTE & 0x3F) | 0x80); // clear SCP and set BBSY (start). must be set at once using MOV
//Pauseus (_I2C_CLOCK_PAUSE_US_);
}

void HI2CStop ()
{
	#if (_I2C_LOG_ == 1)
	DBGPrintf ("\nHI2C stop");
	#endif
//	IIC2.ICSR.BIT.STOP = _CLEAR_; // clear stop
	IIC2.ICCR2.BYTE &= 0x3F;  // clear BBSY and clear SCP (stop). must be set at once using MOV
	IIC2.ICSR.BIT.TEND = _CLEAR_; // transmit end
//Pauseus (_I2C_CLOCK_PAUSE_US_);
	while (IIC2.ICSR.BIT.STOP != _SET_); // wait for stop to complete
//	IIC2.ICCR1.BIT.MST = _I2C_MST_SLAVE_; // select slave mode
	IIC2.ICCR1.BIT.TRS = _I2C_TRS_RECEIVE_; // select receive mode
//Pauseus (_I2C_CLOCK_PAUSE_US_);
}

void HI2CConnect (u8 _bDeviceAddr, u8 _bAddressType, u16 _wIOAddress)
{
	__bHI2CDeviceAddr = _bDeviceAddr;
	#if (_I2C_LOG_ == 1)
	DBGPrintf ("\nHI2C connect %02X %04X %u", _bDeviceAddr, _wIOAddress, _bAddressType);
	#endif
	IIC2.ICCR1.BIT.MST = _I2C_MST_MASTER_; // select master mode
	IIC2.ICCR1.BIT.TRS = _I2C_TRS_TRANSMIT_; // select transmit mode
	HI2CStart ();
	HI2CWriteByte (_bDeviceAddr);
	if (g_bError == _CLEAR_) {
		if (_bAddressType == _I2C_ADDRTYPE_16BIT_)
			HI2CWriteByte (HighByte (_wIOAddress));
		if (g_bError == _CLEAR_)
			if ((_bAddressType == _I2C_ADDRTYPE_16BIT_) || (_bAddressType == _I2C_ADDRTYPE_8BIT_))
				HI2CWriteByte (LowByte (_wIOAddress));
	}
}

void HI2CWriteByte (u8 _bData)
{
	#if (_I2C_LOG_ == 1)
	DBGPrintf ("\nHI2C writebyte %0X", _bData);
	#endif
	while (IIC2.ICSR.BIT.TDRE != _SET_); // wait for transmit register empty
	IIC2.ICDRT = _bData;
	while (IIC2.ICSR.BIT.TEND != _SET_); // wait for transmit to complete
//Pauseus (_I2C_CLOCK_PAUSE_US_);
	g_bError = IIC2.ICIER.BIT.ACKBR;
	if (g_bError != _CLEAR_)
		if (__bHI2CShowAckError)
			DBGPrintf ("\nHI2C ACK err %02X\n\r", __bHI2CDeviceAddr);
}

void HI2CReadByte (u8 *_pbData, u8 _bSendAck)
{
	*_pbData = 0x00;
	if (_bSendAck == _I2C_SEND_NACK_) {
		IIC2.ICIER.BIT.ACKBT = _SET_; // send NACK
		IIC2.ICCR1.BIT.RCVD = _SET_; // disable reception
	}
	*_pbData = IIC2.ICDRR; // read the data
	while (IIC2.ICSR.BIT.RDRF != _SET_); // wait for data receive
	#if (_I2C_LOG_ == 1)
	DBGPrintf ("HI2C readbyte %0X\n", *_pbData);
	#endif
}

void HI2CWrite (u8 _bDeviceAddr, u8 _bAddressType, u16 _wIOAddress, u8 *_pbData, u16 _wWriteCount)
{
	u16 wCountWritten = 0;
	
	if (!__bHI2CInit)
		HI2CInitialize(_I2C_SPEED_400KHZ_);		// Most of the I2C EEPROMS don't want to run at faster than 400khz...
		
	#if (_I2C_LOG_ == 1)
	DBGPrintf ("\nHI2C write %02X %04X %u", _bDeviceAddr, _wIOAddress, _wWriteCount);
	#endif
	g_bError = _CLEAR_;
	HI2CConnect (_bDeviceAddr, _bAddressType, _wIOAddress);
	if (g_bError == _CLEAR_)
	{
		for (wCountWritten=0; wCountWritten<_wWriteCount; wCountWritten++)
		{
			HI2CWriteByte (*_pbData++);
			if (g_bError != _CLEAR_)
				break;
		}
	}
	HI2CStop ();
	#if (_I2C_LOG_ == 1)
	DBGPrintf ("\nHI2C write count %u=%u?", wCountWritten, _wWriteCount);
	#endif
	if (wCountWritten != _wWriteCount)
		g_bError = _SET_;
}

void HI2CRead (u8 _bDeviceAddr, u8 _bAddressType, u16 _wIOAddress, u8 *_pbData, u16 _wReadCount)
{
	u16 wCountRead = 0;
	u8 bData;
	u8 bAck = _I2C_SEND_ACK_;

	if (!__bHI2CInit)
		HI2CInitialize(_I2C_SPEED_400KHZ_);	// lets assume 400khz is as fast as we can go...

	#if (_I2C_LOG_ == 1)
	DBGPrintf ("\nHI2C read %02X %04X %u", _bDeviceAddr, _wIOAddress, _wReadCount);
	#endif
	g_bError = _CLEAR_;
	if (_bAddressType != _I2C_ADDRTYPE_NONE_)
		HI2CConnect (_bDeviceAddr, _bAddressType, _wIOAddress);
	if (g_bError == _CLEAR_) {
		HI2CConnect ((_bDeviceAddr|0x01), _I2C_ADDRTYPE_NONE_, _wIOAddress);
		if (g_bError == _CLEAR_) {
			IIC2.ICSR.BIT.TEND = _CLEAR_; // transmit end
			IIC2.ICCR1.BIT.TRS = _I2C_TRS_RECEIVE_; // select receive mode
			IIC2.ICSR.BIT.TDRE = _CLEAR_; // clear transmit data register empty
			IIC2.ICIER.BIT.ACKBT = _CLEAR_; // send ACK
			bData = IIC2.ICDRR; // dummy read
			while (IIC2.ICSR.BIT.RDRF != _SET_); // wait for data receive
			for (wCountRead=0; wCountRead<_wReadCount; wCountRead++) {
				if (wCountRead == (_wReadCount-1)) // last byte?
					bAck = _I2C_SEND_NACK_;
				HI2CReadByte (_pbData, bAck); // read the data
				_pbData++;
			}
		}
	}
	HI2CStop ();
	#if (_I2C_LOG_ == 1)
	DBGPrintf ("\nHI2C read count %u=%u?", wCountRead, _wReadCount);
	#endif
	if (wCountRead != _wReadCount)
		g_bError = _SET_;
}
//#endif
