//==============================================================================
// Servo_Arc32.cpp - This file contains servo support that hopefully should be similiar
//		to the hservo2 system used in basic.  Will first start off just getting
//		servos to work, will then add in the timed group moves.
//  Not sure if unit base.  g_hservoD.iIndex am thinking in uSs as that is what most people
//	are used to...  Internally we will convert everything to clock tick units...
//
// Version 0.1 - Work in Progress
//
// Major Contributers: Kurt Eckhardt 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================

#include "baplib.h"
#include "Servo.h"
#include "servo_int.h"

#ifdef H8_FAMILY_3687
extern "C" {
typedef void (*PINTHANDLER)(void);
}
extern PINTHANDLER UpdateHandler(int iVect, PINTHANDLER pinth);

const u8 g_aabMuXServoMapTable[4][8] = {
		{2,    7,      10,     15,    18,      20,    28,     25},
		{1,    4,       9,     12,    17,      23,     31,     26},
		{0,    6,       8,     14,     16,      21,     29,     27},
		{3,    5,       11,    13,     19,      22,     30,     24}
};		

// Moved to Servo.cpp as used both in Bap40 and here so to keep pulling code in...
extern volatile unsigned int* apwGRx[];

volatile unsigned char* apbTIOARx[] = {&TZ0.TIORA.BYTE, &TZ0.TIORA.BYTE, &TZ0.TIORC.BYTE, &TZ0.TIORC.BYTE, 
										&TZ1.TIORA.BYTE, &TZ1.TIORA.BYTE, &TZ1.TIORC.BYTE, &TZ1.TIORC.BYTE};

volatile unsigned char abTOARxMask[] = {0x7, 0x70, 0x7, 0x70, 0x7, 0x70, 0x7, 0x70};

//*****************************************************************************
// ARC32 version - HSERVO2 type functionality...
//*****************************************************************************

//=============================================================================
// TIMERZ0-Arc32() - Main timerw function that is used to handle the background processing
// of the servo sub-system
//=============================================================================

extern "C" void TIMERZ0_ARC32() __attribute__ ((interrupt_handler));
extern "C" void TIMERZ0_ARC32()
{
	register u8 bChannels;
	// See why we woke up for - Do the easy ones first. If we get a GRA-D match, than we should have already
	// set the output pin high, simply set the next match to something just before the counter overflow
	// and set up the pin to go back low at that point...

	g_hservoD.bMask = 1;
	
	g_hservoD.bWhichIntsToProcess = TZ0.TSR.BYTE & TZ0.TIER.BYTE & 0xf;	// Only process the 4 bits here 1

	DSERVOTOGGLE(40);		// BUGBUG:: For debug servo stuff.

	// Note the array holds both Tz0 and TZ1 info Only process the TZ0 stuff here
	for (g_hservoD.iIndex=0; g_hservoD.iIndex < 4; g_hservoD.iIndex++) {
		if (g_hservoD.bWhichIntsToProcess & g_hservoD.bMask) {
			DSERVOTOGGLE(41);		// BUGBUG:: For debug servo stuff.
			*(apwGRx[g_hservoD.iIndex]) = 0xffff;	// Set the appropriate register GRa-GRd
			
			// Update TIORA/TIORC???  
			*(apbTIOARx[g_hservoD.iIndex]) = (*(apbTIOARx[g_hservoD.iIndex]) & ~abTOARxMask[g_hservoD.iIndex]) | (abTOARxMask[g_hservoD.iIndex] & 0x11);	// will only set the appropriate nibble to 1...
		}
		g_hservoD.bMask <<= 1;
	}
	TZ0.TIER.BYTE &= ~g_hservoD.bWhichIntsToProcess;
	TZ0.TSR.BYTE &= ~g_hservoD.bWhichIntsToProcess;

	
	// OK the main work horse.
	if (TZ0.TSR.BIT.OVF) {					// Use the real bit here, we restricted what we cleared up above!
		TZ0.TSR.BIT.OVF = 0;				// Clear the overflow flag now!.
		g_hservoD.ulTimerOverflow++;		// timer overflow counter.

		// OK, before we switch to a new channel, we may need to drive the signals low that have
		// a backing IO pin as they do not have a pull-down resistor anymore or it may not be
		// strong enough to counter pull-up resistors in some servos...
		for (g_hservoD.iIndex=0; (g_hservoD.iIndex< NUMTIMERCOMPAREMATCH) && (g_hservoD.abActivePins[g_hservoD.iIndex] != 0xff); g_hservoD.iIndex++) {		// Only process the number of channels that were used...
			g_hservoD.ppiTemp = &((*g_pPortTable)[g_hservoD.abActivePins[g_hservoD.iIndex]]);		// Get the port item for this one.
			PPI_OUTPUT(g_hservoD.ppiTemp);									// make sure the backing pin is output
			PI_LOW((*g_hservoD.ppiTemp));										// and forced low
			g_hservoD.abActivePins[g_hservoD.iIndex] = 0xff; // only process once...
		}

		g_hservoD.bChannel++;				// increment the channel.
		if (g_hservoD.bChannel > 5) // handle wrap
			g_hservoD.bChannel = 0;
		if (g_hservoD.bChannel <= 3) {
			// Setup for the next channel.
			// Note sure of order...
			if (g_hservoD.bChannel & 2)
				PI_HIGH(g_hservoD.piChHigh);
			else
				PI_LOW(g_hservoD.piChHigh);

			if (g_hservoD.bChannel & 1)
				PI_HIGH(g_hservoD.piChLow);
			else
				PI_LOW(g_hservoD.piChLow);
			
			//setpin(47, (g_hservoD.bChannel >> 1));	// set it to high order bit
			//setpin(46, (g_hservoD.bChannel & 1));		// set to low order bit
			TZ0.TIORA.BYTE = 0x0;		// Tell all channels we are not doing any output
			TZ0.TIORC.BYTE = 0x0;
			TZ1.TIORA.BYTE = 0x0;
			TZ1.TIORC.BYTE = 0x0;
			TZ0.TSR.BYTE = 0;	// clear all status
			TZ1.TSR.BYTE = 0;
			TZ0.TIER.BYTE &= 0xf0;	// clear out Match on channels A-D
			TZ1.TIER.BYTE = 0x0;	// Again clear out all on other...
			
#define A32_TZF	70
			bChannels = 0;		// number of servos that will be active in this pass
			g_hservoD.bMask = 1;			// Save from doing counted shifts...
			for (g_hservoD.iIndex=0; g_hservoD.iIndex < 8; g_hservoD.iIndex++) {
				g_hservoD.iServo = g_aabMuXServoMapTable[g_hservoD.bChannel][g_hservoD.iIndex];
				if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
					IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
					*(apwGRx[g_hservoD.iIndex]) = (65535 - A32_TZF) - g_hservoD.ahsi[g_hservoD.iServo].wPW;  // 

					// Table drive this part?
					if (g_hservoD.iIndex < 4) {
						TZ0.TSR.BYTE &= ~g_hservoD.bMask;		// Make sure the flag is clear
						TZ0.TIER.BYTE |= g_hservoD.bMask;		// enable that interrupt
					} else {
						TZ1.TSR.BYTE &= ~g_hservoD.bMask;		// Make sure the flag is clear
						TZ1.TIER.BYTE |= g_hservoD.bMask;		// enable that interrupt
					}
					*(apbTIOARx[g_hservoD.iIndex]) = *(apbTIOARx[g_hservoD.iIndex]) | (abTOARxMask[g_hservoD.iIndex] & 0x22);	// will only set the appropriate nibble to 2 - Don't need to clear was done above...
					DSERVOTOGGLE(42);		// BUGBUG:: For debug servo stuff.

					// Obviously only those servo pins that have actual io lines connected to them can be switched from output to input
					// so check that now and only remember those that are this way...
					g_hservoD.ppiTemp = &((*g_pPortTable)[g_hservoD.iServo]);		// Get the port item for this one.
					if (g_hservoD.ppiTemp->pPDR) {									// this pin does have a backing pin
						g_hservoD.abActivePins[bChannels++] = g_hservoD.iServo;		// remember which pin we are using
						PPI_INPUT(g_hservoD.ppiTemp);									// make sure the backing pin floats for now
					}
				}
				g_hservoD.bMask <<= 1;
				if (g_hservoD.bMask == (1<<4))			// Handle switch to TZ1...
					g_hservoD.bMask = 1;
			}

		} else {
			// Later: Add analog pin reading 	
		}
	}
}

//=============================================================================
// TIMERZ1_ARC32 - For now the only timerZ1 function.  Only needs to handle
// the GRA-GRD interrupt as timerz0 will do all of the setup for overflow...
//
//=============================================================================
extern "C" void TIMERZ1_ARC32() __attribute__ ((interrupt_handler));
extern "C" void TIMERZ1_ARC32()
{
	g_hservoD.bMask = 1;
	g_hservoD.bWhichIntsToProcess = TZ1.TSR.BYTE & TZ1.TIER.BYTE;	// Only process the bits that are enabled...

	DSERVOTOGGLE(41);		// BUGBUG:: For debug servo stuff.
	// Note the array holds both Tz0 and TZ1 info so start at TZ1
	for (g_hservoD.iIndex=4; g_hservoD.iIndex < 8; g_hservoD.iIndex++) {
		if (g_hservoD.bWhichIntsToProcess & g_hservoD.bMask) {
			*(apwGRx[g_hservoD.iIndex]) = 0xffff;	// Set the appropriate register GRa-GRd
			
			// Update TIORA/TIORC???  
			*(apbTIOARx[g_hservoD.iIndex]) = (*(apbTIOARx[g_hservoD.iIndex]) & ~abTOARxMask[g_hservoD.iIndex]) | (abTOARxMask[g_hservoD.iIndex] & 0x11);	// will only set the appropriate nibble to 1...
		}
		g_hservoD.bMask <<= 1;
	}
	TZ1.TIER.BYTE &= ~g_hservoD.bWhichIntsToProcess;
	TZ1.TSR.BYTE &= ~g_hservoD.bWhichIntsToProcess;
}

#endif // H9_FAMILY_3687



