//==============================================================================
// HSIO.c - Hardware Serial port(s)
//
// Version 0.1 - Work in Progress
//
// Major Contributers: Kurt Eckhardt 
// 		Robert V Mahoney Jr
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================

#include "BapLib.h"
#ifdef __cplusplus
extern "C" {
#endif

#include "hsio.h"


//------------------------------------------------------------------------------
// Defines
//------------------------------------------------------------------------------
#define _BA_PIN_MODE_IO_				0
#define _BA_PIN_MODE_FUNCTION_			1

#define _BA_PIN_MODE_INPUT_				0
#define _BA_PIN_MODE_OUTPUT_			1

#define _BA_SCI3_MODE_ASYNC_			0
#define _BA_SCI3_MODE_SYNC_				1

#define _BA_SCI3_MULPROC_DISABLE_		0
#define _BA_SCI3_MULPROC_ENABLE_		1

#define _BA_SCI3_CLOCK_DIV_0_			0
#define _BA_SCI3_CLOCK_DIV_4_			1
#define _BA_SCI3_CLOCK_DIV_16_			2
#define _BA_SCI3_CLOCK_DIV_64_			3

#define _BA_SCI3_BAUDGEN_INTERNAL_		0

//------------------------------------------------------------------------------
// Type definitions
//------------------------------------------------------------------------------
#define	HSERQ_SIZE		64	// Assumed to be a power of 2!
typedef struct _hserq
{	
	volatile  u8		iStart;		// start pointer
	volatile u8		iEnd;		// end
	u8		abBuf[HSERQ_SIZE];			// our actual buffer
} HSERQ, *PHSERQ;


typedef struct _hportinfo
{
	volatile struct st_sci3 * psci3;
	HSERQ	hserqo;		// output queue
	HSERQ	hserqi;		// input queue
	boolean fInit;		// has the queue been init yet...
} HPORTINFO, *PHPORTINFO;

//------------------------------------------------------------------------------
// Globals
//------------------------------------------------------------------------------

//HSERQ	g_HSERQHSO;	// output queue for hardware serial port 1
//HSERQ	g_HSERQHSI;	// input queue
#ifdef H8_FAMILY_3694
HPORTINFO g_ahportinfo[1];	// only have 1 hardware serial port
#endif
#ifdef H8_FAMILY_3687  // Pro40 and arc32
HPORTINFO g_ahportinfo[2];
//HSERQ	g_HSERQHSO_2;	// output queue for hardware serial port 1
//HSERQ	g_HSERQHSI_2;	// input queue
#endif

#define CHSERIALPORTS 	(sizeof(g_ahportinfo)/sizeof(HPORTINFO))


//------------------------------------------------------------------------------
// Forward references
//------------------------------------------------------------------------------

//=============================================================================
// Simple Queue functions for adding characters to be processed by Hserin/Hserout
//=============================================================================
inline void ClearQueue(PHSERQ phq) __attribute__((always_inline));
void ClearQueue(PHSERQ phq)
{
	// Initialize the queue to be empty.
	phq->iStart = 0;
	phq->iEnd = 0;
}


// Add a byte to the queue. - was bool but no one checking return value
inline void AddByteToQueue(PHSERQ phq, u8 b, bool fWait) __attribute__((always_inline));

/*bool*/ void  AddByteToQueue(PHSERQ phq, u8 b, bool fWait)
{
	register u8	iOutNext; 
	
	// Calculate the address to save away the byte in.
	iOutNext = (phq->iEnd + 1) & (HSERQ_SIZE-1);		// Again assume power of 2

	while (iOutNext == phq->iStart) {
		// while the queue is full
		if (!fWait)	
			return /*false */;
	}
	phq->abBuf[phq->iEnd] = b;
	phq->iEnd = iOutNext;
	
	return /*true*/;
}

inline word RetrieveByteFromQueue(PHSERQ phq) __attribute__((always_inline));
word RetrieveByteFromQueue(PHSERQ phq)
{
	word	wRet;
	if (phq->iStart == phq->iEnd)
		return 0xffff;
		
	wRet = phq->abBuf[phq->iStart];

	// Increment our index to where the next byte should be from, handle wrap around	
	phq->iStart = (phq->iStart + 1) & (HSERQ_SIZE-1);
	return wRet;
}




//------------------------------------------------------------------------------
// enablehserial - setup the first serial port
//------------------------------------------------------------------------------
bool enablehserial (u8 bPortnum)
{
	// We 1 index our parameter so decrement first before we use it...
	if (--bPortnum >= CHSERIALPORTS)
		return 0;		// 

	// first make sure we init our pointers to the hardware port informatino...
	g_ahportinfo[0].psci3 = &SCI3;
#ifdef H8_FAMILY_3687
	g_ahportinfo[1].psci3 = &SCI3_2;
#endif
	g_ahportinfo[bPortnum].fInit = 1; // say it is init.
	ClearQueue(&g_ahportinfo[bPortnum].hserqo);	
	ClearQueue(&g_ahportinfo[bPortnum].hserqi);	
	
#ifdef BASICATOMPRO28
	// Make sure the other H8 IO pin connected to our pin is set to input, which should let it float
	// On Bap40/arc 32 They are their own pin...
	input(14);
	input(15);
#endif

	// Could use our pointers above for most of this...
	if (bPortnum == 0) {
		// set TXD as TXD
		IO.PMR1.BIT.TXD = _BA_PIN_MODE_FUNCTION_;
		// set RXD as RXD
		SCI3.SCR3.BIT.RE = _BA_PIN_MODE_FUNCTION_;
		// set SCK3 as IO pin
		SCI3.SCR3.BIT.CKE = _BA_PIN_MODE_IO_;
		SCI3.SMR.BIT.COM = _BA_PIN_MODE_IO_;
		// set SCK3 as ASYNC pin
		SCI3.SMR.BIT.COM = _BA_SCI3_MODE_ASYNC_;
		SCI3.SMR.BIT.MP = _BA_SCI3_MULPROC_DISABLE_;
		SCI3.SCR3.BIT.RIE = _FALSE_;		// enable interrupts when we receive a character or error
		SCI3.SCR3.BIT.TE = _FALSE_;
		SCI3.SCR3.BIT.RE = _FALSE_;
		SCI3.SCR3.BIT.MPIE = _FALSE_;
		SCI3.SCR3.BIT.TEIE = _FALSE_;	// For now don't need to enable interrupt when transmit complete...
		SCI3.SCR3.BIT.CKE = _BA_SCI3_BAUDGEN_INTERNAL_;
		// enable receiver
		SCI3.SCR3.BIT.RE = _TRUE_;
		SCI3.SCR3.BIT.TE = _TRUE_;
#ifdef H8_FAMILY_3687
	} else if (bPortnum == 1) {
	
		// On Bap40 on pins 14 and 15 and share pin with normal IO pin.  Set the normal IO pin
		// to input such that it will float...
		if (g_bWhichBap == H8_3687_BAP40) {
			input(14);
			input(15);
		}
	
		// set TXD_2 as TXD
		IO.PMR1.BIT.TXD2 = _BA_PIN_MODE_FUNCTION_;
		// set RXD_2 as RXD
		SCI3_2.SCR3.BIT.RE = _BA_PIN_MODE_FUNCTION_;
		// set SCK3 as IO pin
		SCI3_2.SCR3.BIT.CKE = _BA_PIN_MODE_IO_;
		SCI3_2.SMR.BIT.COM = _BA_PIN_MODE_IO_;
		// set SCK3 as ASYNC pin
		SCI3_2.SMR.BIT.COM = _BA_SCI3_MODE_ASYNC_;
		SCI3_2.SMR.BIT.MP = _BA_SCI3_MULPROC_DISABLE_;
		SCI3_2.SCR3.BIT.TIE = _FALSE_;
		SCI3_2.SCR3.BIT.RIE = _FALSE_;
		SCI3_2.SCR3.BIT.TE = _FALSE_;
		SCI3_2.SCR3.BIT.RE = _FALSE_;
		SCI3_2.SCR3.BIT.MPIE = _FALSE_;
		SCI3_2.SCR3.BIT.TEIE = _FALSE_;
		SCI3_2.SCR3.BIT.CKE = _BA_SCI3_BAUDGEN_INTERNAL_;
		// enable transmitter
		SCI3_2.SCR3.BIT.TE = _TRUE_;
		// enable receiver
		SCI3_2.SCR3.BIT.RE = _TRUE_;
#endif
	}
	return 1;
}


//-----------------------------------------------------------------------------
// HSETHSERIAL - Initialize the 1st hardware serial port
//-----------------------------------------------------------------------------
bool sethserial (u8 bPortnum, u32 _lbaudrate, u8 _bdatabits, u8 _bparity, u8 _bstopbits)
{
	word n;
	
	if (--bPortnum >= CHSERIALPORTS)
		return 0;		// 

	// Make sure we were init.
	if (!g_ahportinfo[bPortnum].fInit)
		enablehserial(bPortnum);

	g_ahportinfo[bPortnum].psci3->SMR.BIT.CHR = _bdatabits;
	g_ahportinfo[bPortnum].psci3->SMR.BIT.PE = (_bparity >> 1);
	g_ahportinfo[bPortnum].psci3->SMR.BIT.PM = (_bparity & 0x1);
	g_ahportinfo[bPortnum].psci3->SMR.BIT.STOP = _bstopbits;

	// Lets calculate it, not hard code it...
	n = (ulong)F_CPU/_lbaudrate;		// See which divisor we should use
	// see if we can use DIV_0
	if (n < (255*32)) {
		g_ahportinfo[bPortnum].psci3->SMR.BIT.CKS = _BA_SCI3_CLOCK_DIV_0_;
		g_ahportinfo[bPortnum].psci3->BRR = (n+16)/32 - 1;
	} else {
		// I think we won't need to go to any higher
		g_ahportinfo[bPortnum].psci3->SMR.BIT.CKS = _BA_SCI3_CLOCK_DIV_4_;
		g_ahportinfo[bPortnum].psci3->BRR = (n+64)/128 - 1;
	}
	
	// Ok we are now ready to receive something...
	//g_ahportinfo[bPortnum].psci3->SCR3.BIT.TIE = _TRUE_;		// enable interrupt for transmit...
	//g_ahportinfo[bPortnum].psci3->SCR3.BIT.TEIE = _TRUE_;	// enable interrupt for transmit end
	g_ahportinfo[bPortnum].psci3->SCR3.BIT.RIE = _TRUE_;		// enable interrupts when we receive a character or error
	enable();						// make sure interupts are enabled...
	return 1;
}



//-----------------------------------------------------------------------------
// HSEROUT - output a string
//-----------------------------------------------------------------------------
bool hserout (u8 bPortnum, u8 *_psdata, u16 _wcount)
{
	u16 wbyteloop;

	if (--bPortnum >= CHSERIALPORTS)
		return 0;		// 

	for (wbyteloop=0; wbyteloop<_wcount; wbyteloop++) {
		u8	b = *_psdata++;

		// See if the transmitter is active.
		if (g_ahportinfo[bPortnum].psci3->SSR.BIT.TDRE) {			// If the transmitter is not active
			// enable transmitter
			g_ahportinfo[bPortnum].psci3->TDR = b;					// and put the first byte out on the output register...
			g_ahportinfo[bPortnum].psci3->SCR3.BIT.TIE = _TRUE_;		// enable interrupts when ready for next byte to transmit
		} else {
			// It was active when we started...
			AddByteToQueue(&g_ahportinfo[bPortnum].hserqo, b, true);

			if (g_ahportinfo[bPortnum].psci3->SSR.BIT.TDRE) {				// Says I can stuff a byte out...
					word w;
				if ((w=RetrieveByteFromQueue(&g_ahportinfo[bPortnum].hserqo)) != 0xffff) {		
					g_ahportinfo[bPortnum].psci3->TDR = w & 0xff;			// and put the first byte out on the output register...
					g_ahportinfo[bPortnum].psci3->SCR3.BIT.TIE = _TRUE_;		// enable interrupts when ready for next byte to transmit
				}
			}
		}
	}
	return 1;
}


//-----------------------------------------------------------------------------
// HSERIN - Retrieve bytes of data from the hardware serial port.  Will return
//			the number of bytes actually returned.
//-----------------------------------------------------------------------------
u16 hserin (u8 bPortnum, u32 _ltimeout, u8 *_psdata, u16 _rcount, u16 _cterminator)
{
	u16 wbyteloop;
	u8 *_psdataIN = _psdata;
	u32 lTOCnt;
	word	w;

	if (--bPortnum >= CHSERIALPORTS)
		return 0;		// 

	// rough calculation is the loop while the queue is empty takes about
	// 104 clocks or on Bap28 about 6.5us ... Could speed this up...
	if (_ltimeout != (u32)-1)
#ifdef BASICATOMPRO28
		_ltimeout = (_ltimeout * 2L)/13L;		// Need to update for bap 40...
#else		
		_ltimeout = (_ltimeout * 5L)/26L;		// Updated for 20mhz
#endif
	for (wbyteloop=0; wbyteloop<_rcount; wbyteloop++) {
		lTOCnt = _ltimeout;
		while ((w = RetrieveByteFromQueue(&g_ahportinfo[bPortnum].hserqi)) == 0xffff) 	{
			// Check for timeout...  First pass will be real primitive
			// maybe should integrate the TimerA code to be used here...
			if ((lTOCnt != (u32)-1) && (--lTOCnt == 0))
				goto Timeout;
		}

		// check for terminator.  - changed to word so we don't
		// have characters we can not check for...
		*_psdata++ = w & 0xff;		// save away the character and increment our output pointer.
		if ((_cterminator>>8 == 0) && (w == _cterminator))
			break;
	}
	
	// return how many characters that 
Timeout:	
	return (u16)(_psdata - _psdataIN);
}


//-----------------------------------------------------------------------------
// HSERSTAT
//-----------------------------------------------------------------------------
bool hserstat(u8 bPortnum, HSERSTAT_OPTIONS ho)  // ? - would it be better to simply have a function for each???
{
	if (--bPortnum >= CHSERIALPORTS)
		return 0;		// 

	switch (ho)
	{
		case HSERSTAT_CLEAR_INPUT_BUFFER:
			ClearQueue(&g_ahportinfo[bPortnum].hserqi);
			return true;

		case HSERSTAT_CLEAR_BOTH_BUFFERS:
			ClearQueue(&g_ahportinfo[bPortnum].hserqi);
			// fall through			
		case HSERSTAT_CLEAR_OUTPUT_BUFFER:
			ClearQueue(&g_ahportinfo[bPortnum].hserqo);
			return true;
			
		case HSERSTAT_INPUT_AVAILABLE:
			return (g_ahportinfo[bPortnum].hserqi.iStart != g_ahportinfo[bPortnum].hserqi.iEnd);
			
		case HSERSTAT_OUTPUT_EMPTY:
			return (g_ahportinfo[bPortnum].hserqo.iStart == g_ahportinfo[bPortnum].hserqo.iEnd);
	}

	return false;	// unknown command
}



//-----------------------------------------------------------------------------
// HSERAVAIL
//-----------------------------------------------------------------------------
byte hseravail(u8 bPortnum)  // ? - would it be better to simply have a function for each??? yes
{
	if (--bPortnum >= CHSERIALPORTS)
		return 0;		// 

	if (g_ahportinfo[bPortnum].hserqi.iStart == g_ahportinfo[bPortnum].hserqi.iEnd)
		return 0;		// It is empty;
	return (HSERQ_SIZE +  g_ahportinfo[bPortnum].hserqi.iEnd -  g_ahportinfo[bPortnum].hserqi.iStart) % HSERQ_SIZE;
}


//=============================================================================
// Interrupt handler for SCI3 (UART)
//=============================================================================
extern "C" void SCI3INT() __attribute__ ((interrupt_handler));
extern "C" void SCI3INT()
{
	word	w;
	// OK we need to find out why we were interrupted...
	if (SCI3.SSR.BIT.TDRE) 	{
		// Transmit Data register empty
		if ((w=RetrieveByteFromQueue(&g_ahportinfo[0].hserqo)) != 0xffff)
			SCI3.TDR = w & 0xff;
		else
			SCI3.SCR3.BIT.TIE = _FALSE_;		// Disable this interrupt when ready for next byte to transmit
			
	}

	if (SCI3.SSR.BIT.OER) {
		// Data overrun error
		SCI3.SSR.BIT.OER = 0; 		// should maybe do something with error conditions?
	}

	if (SCI3.SSR.BIT.RDRF) {
		// Receive data Register Full
		// Should maybe do something if our queue was full and we errored...
		w = SCI3.RDR;
		AddByteToQueue(&g_ahportinfo[0].hserqi,  w &0xff, false);
	}

	if (SCI3.SSR.BIT.FER) {
		// data framing error
		SCI3.SSR.BIT.FER = 0;
	}
	
	if (SCI3.SSR.BIT.PER) {
		// data parity error
		SCI3.SSR.BIT.PER = 0;
	}
}

#ifdef H8_FAMILY_3687

//=============================================================================
// Interrupt handler for SCI3_2 (UART)
//=============================================================================
extern "C" void SCI3_2INT() __attribute__ ((interrupt_handler));
extern "C" void SCI3_2INT()
{
	word	w;
	// OK we need to find out why we were interrupted...
	if (SCI3_2.SSR.BIT.TDRE) 	{
		// Transmit Data register empty
		if ((w=RetrieveByteFromQueue(&g_ahportinfo[1].hserqo)) != 0xffff)
			SCI3_2.TDR = w & 0xff;
		else
			SCI3_2.SCR3.BIT.TIE = _FALSE_;		// Disable this interrupt when ready for next byte to transmit
			
	}

	if (SCI3_2.SSR.BIT.OER) {
		// Data overrun error
		SCI3_2.SSR.BIT.OER = 0; 		// should maybe do something with error conditions?
	}

	if (SCI3_2.SSR.BIT.RDRF) {
		// Receive data Register Full
		// Should maybe do something if our queue was full and we errored...
		//toggle(20);		// debug Pro40
		w = SCI3_2.RDR;
		AddByteToQueue(&g_ahportinfo[1].hserqi,  w &0xff, false);
	}

	if (SCI3_2.SSR.BIT.FER) {
		// data framing error
		SCI3_2.SSR.BIT.FER = 0;
	}
	
	if (SCI3_2.SSR.BIT.PER) {
		// data parity error
		SCI3_2.SSR.BIT.PER = 0;
	}
}
#endif



#ifdef __cplusplus
}
#endif
