//==============================================================================
// PS2SUP - Low level PS2 support
// Warning: this may be removed as I have the C++ class I like better!
//
// Version 0.1 - Work in Progress
//
// Major Contributers: Kurt Eckhardt 
// 		Robert V Mahoney Jr
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================
//=============================================================================
//=============================================================================

// [Include files]
#include "BapLib.h"
#include "ps2sup.h"

//=============================================================================
// Global - Local to this file only...
//=============================================================================

PORTITEM			g_piCMD;
PORTITEM			g_piDAT;
PORTITEM			g_piSEL;
PORTITEM			g_piCLK;

//-----------------------------------------------------------------------------
// Main PS2 input functions
//-----------------------------------------------------------------------------
void PS2_Init(u8 bDat, u8 bCmd, u8 bSel, u8 bClk)  // pass in which pins you wish to use
{
	DBGPrintf("PS2 Init\r");
   	input(bDat, true);  			// make the dat line input But also enable pull-up
	output(bCmd);				// Make command pin be output
	output(bSel);				// init the select pin
	output(bClk);				// Make clock be output


	// First pass see if using our array of Porttable items with constants will generate same code as local copy...
	g_piCMD = (*g_pPortTable)[bCmd];		
	g_piDAT = (*g_pPortTable)[bDat];
	g_piSEL = (*g_pPortTable)[bSel];
	g_piCLK = (*g_pPortTable)[bClk];

	PI_HIGH(g_piSEL);					// Initialize to high...
};

// PSx controller communication function.
// send a byte on the command line and receive one on the data line.
// needs Attention pin to have gone low before called to activate controller.
byte gameByte(register byte command)
{
        short int i ;
		short int j;
        // pause(1);
        register short int data = 0x00;                                   // clear data variable to save setting low bits later.
        for(i=0;i<8;i++)
        {
			if (command & 0x1)
				PPI_HIGH(&g_piCMD);		//sbi(PORTK, PScommand);      // bit bang "command" out on PScommand wire.
            else 
				PPI_LOW(&g_piCMD);		//cbi(PORTK, PScommand);
            PPI_LOW(&g_piCLK);			//cbi(PORTK, PSclock);                             // CLOCK LOW
			for (j=5; j > 0; j--)
				;
            if ( PI_ISHIGH(g_piDAT))		//if((PINK & _BV(PSdata))) 
				data |= (1<<i);		// sbi(data, i); 
            PPI_HIGH(&g_piCLK);			//sbi(PORTK, PSclock);                             // CLOCK HIGH
			command = command >> 1;
        }
        PPI_HIGH(&g_piCMD);				//sbi(PORTK, PScommand);

//        delay_us(20);                                            // wait for ACK to pass.
		for (j=20; j > 0; j--)
				;
		

        return(data);
}

void _PS2_ShiftOut(unsigned char *pb, int cb)
{
	while (cb)
	{
		gameByte(*pb++);
		cb--;
	}
}

byte PS2_Query(unsigned char *paData, int cb)
{
	int iLoopCnt;
	unsigned char bMode;

	//pause(100);		// see if timing is keeping the two from talking	
	// Hack to not have this loop forever in case of error
	for (iLoopCnt = 0; iLoopCnt < 10; iLoopCnt++)
	{
		//spiDeviceSelect(PS2_device, 1);
		PPI_LOW(&g_piSEL);
		
		
//		shiftout CMD,CLK,FASTLSBPRE,[$1\8]
		gameByte(0x1);
		
		//shiftin DAT,CLK,FASTLSBPOST,[DS2Mode\8]
		bMode = gameByte(0x42);

		if ((bMode & 0xf0) == 0x70)
		{
			// we are in an analog mode so continue to read in the extra bytes.
			gameByte(0);		// ignore first byte which should return 0x5a
			while (cb > 0)
			{
				*paData++ = gameByte(0);
				cb--;
			}

			// And clear our select line...
			PPI_HIGH(&g_piSEL);
		
			return 1;
		}

		//DBGPrintf("PS2 Query - Not Analog %x \r", bMode);
		PPI_HIGH(&g_piSEL);
		pause(1);

		// Not initialized properly, try to send out the correct init sequences.
		PPI_LOW(&g_piSEL);
//		shiftout CMD,CLK,FASTLSBPRE,[$1\8,$43\8,$0\8,$1\8,$0\8] ;CONFIG_MODE_ENTER
		static unsigned char _aps2_2a[] = {0x01, 0x43, 0x0, 0x1, 0x0};
		_PS2_ShiftOut(_aps2_2a, sizeof(_aps2_2a));
		PPI_HIGH(&g_piSEL);
		pause(1);
	
		PPI_LOW(&g_piSEL);
//		shiftout CMD,CLK,FASTLSBPRE,[$01\8,$44\8,$00\8,$01\8,$03\8,$00\8,$00\8,$00\8,$00\8] ;SET_MODE_AND_LOCK
		static unsigned char _aps2_3[] = {0x01, 0x44, 0, 1, 3, 0, 0, 0, 0};
		_PS2_ShiftOut(_aps2_3, sizeof(_aps2_3));
		PPI_HIGH(&g_piSEL);
		pause(1);
	
		//;pin_low(g_piSEL);
		//;shiftout CMD,CLK,FASTLSBPRE,[$01\8,$4D\8,$00\8,$00\8,$01\8,$FF\8,$FF\8,$FF\8,$FF\8] ;VIBRATION_ENABLE
		//;pin_high(g_piSEL);
		//;clockWaitms(1);

		//pin_low(g_piSEL);
        //shiftout CMD,CLK,FASTLSBPRE,[$01\8,$4F\8,$00\8,$FF\8,$FF\8,$03\8,$00\8,$00\8,$00\8] ;SET_DS2_NATIVE_MODE
		//static unsigned char _aps2_4[] = {0x01, 0x4F, 0, 0xff, 0xff, 3, 0, 0, 0};
		//_PS2_ShiftOut(_aps2_4, sizeof(_aps2_4));
		//pin_high(g_piSEL);
		//clockWaitms(1);
	
		PI_LOW(g_piSEL);
//		shiftout CMD,CLK,FASTLSBPRE,[$01\8,$43\8,$00\8,$00\8,$5A\8,$5A\8,$5A\8,$5A\8,$5A\8] ;CONFIG_MODE_EXIT_DS2_NATIVE
		static unsigned char _aps2_5[] = {0x01, 0x43, 0, 0, 0x5a, 0x5a, 0x5a, 0x5a, 0x5a};
		_PS2_ShiftOut(_aps2_5, sizeof(_aps2_5));
		PI_HIGH(g_piSEL);
//		clockWaitms(1);
	
//		pin_low(g_piSEL);
//		shiftout CMD,CLK,FASTLSBPRE,[$01\8,$43\8,$00\8,$00\8,$00\8,$00\8,$00\8,$00\8,$00\8] ;CONFIG_MODE_EXIT
////		static unsigned char _aps2_6[] = {0x01, 0x43, 0, 0, 0, 0, 0, 0, 0};
//		_PS2_ShiftOut(_aps2_6, sizeof(_aps2_6));
//		pin_high(g_piSEL);
		pause(144); //nap 4
	}
	
	// If we got here it means that we tried a number of times and it failed, so set up a set of default values and return.
	
	paData[0] = 255;
	paData[1] = 255; 
	paData[2] = 128;
	paData[3] = 128;
	paData[4] = 128;
	paData[5] = 128;
	
	return 0;
}

