//==============================================================================
// Servo.cpp - This file contains servo support that hopefully should be similiar
//		to the hservo system used in basic.  Will first start off just getting
//		servos to work, will then add in the timed group moves.
//  Not sure if unit base.  g_hservoD.iIndex am thinking in uSs as that is what most people
//	are used to...  Internally we will convert everything to clock tick units...
//
// Version 0.1 - Work in Progress
//
// Major Contributers: Kurt Eckhardt 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================


#include "baplib.h"
#include "Servo.h"
#include "servo_int.h"

HSERVO_DATA g_hservoD;

// Defined here so common for both Bap40 and Arc32...
#ifdef H8_FAMILY_3687
volatile unsigned int* apwGRx[] = {&TZ0.GRA, &TZ0.GRB, &TZ0.GRC, &TZ0.GRD, &TZ1.GRA, &TZ1.GRB, &TZ1.GRC, &TZ1.GRD};
#endif

#ifdef BASICATOMPRO28
//=============================================================================
// TIMERW() - Main timerw functino that is used to handle the background processing
// of the servo sub-system
//=============================================================================
extern "C" void TIMERW() __attribute__ ((interrupt_handler));
extern "C" void TIMERW()
{
	// We work to speed up some more... 
	// See why we woke up for
	DSERVOHIGH(0);		// BUGBUG:: For debug servo stuff.
	if (TW.TSRW.BIT.OVF) {
		DSERVOTOGGLE(1);		// BUGBUG:: For debug servo stuff.
#define SERVO_TIMING_FUDGE 70

		// If no servos are currently active we can now bail as we have
		// set all of the previously active servos back to low...
		if (!g_hservoD.cServosActive) {
			TW.TSRW.BYTE = 0;		// turn off all of the status bytes.
			TW.TIERW.BYTE = 0;		// Turn off all interrupts
			goto Exit;
		}

		// Clock over run.  we are ready to setup for the next pass.
		// setup TCNT such that we interrupt every 4mS.  For each 4ms we handle 1 servo per GRx value so
		// 4 servos.  So every 5 times through is one complete cycle which equals 20ms
		TW.TCNT = 1536-90;				// set such that 65536-1536=64000tics per interrupt/16MperHz = 4000uS= 4ms (fudge)
		g_hservoD.iServo++; 
		if (g_hservoD.iServo >= NUMSERVOS) {
			g_hservoD.iServo = 0;
		}
		
		g_hservoD.iServoGroupStart = g_hservoD.iServo;
			
		// Now lets start up to see if which servo we need to process in this pass
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
			
			// servo is active so lets get all of the timers and the like setup
			TW.GRA = TW.TCNT + g_hservoD.ahsi[g_hservoD.iServo].wPW - SERVO_TIMING_FUDGE;  // off by 1 but should work
			PI_HIGH((*g_pPortTable)[g_hservoD.iServo]);	// So set this servo pin high now...
			TW.TIERW.BIT.IMIEA = 1;		// enable that interrupt
				
		}
		// Channel B
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TW.GRB = TW.TCNT + g_hservoD.ahsi[g_hservoD.iServo].wPW - SERVO_TIMING_FUDGE;  // off by 1 but should work
			PI_HIGH((*g_pPortTable)[g_hservoD.iServo]);	// Like A
			TW.TIERW.BIT.IMIEB = 1;		// enable that interrupt
		}

		// Channel C
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TW.GRC = TW.TCNT + g_hservoD.ahsi[g_hservoD.iServo].wPW - SERVO_TIMING_FUDGE;  // off by 1 but should work
			PI_HIGH((*g_pPortTable)[g_hservoD.iServo]);	// Like A
			TW.TIERW.BIT.IMIEC = 1;		// enable that interrupt
		}
		
		// Channel D
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TW.GRD = TW.TCNT + g_hservoD.ahsi[g_hservoD.iServo].wPW - SERVO_TIMING_FUDGE;  // off by 1 but should work
			PI_HIGH((*g_pPortTable)[g_hservoD.iServo]);	// Like A
			TW.TIERW.BIT.IMIED = 1;		// enable that interrupt
		}

		TW.TSRW.BYTE = 0;								// Clear all of the status bits
		goto Exit;
	}

	// Lets see which of the 4 channels we woke up for.  Lets try with pre-generated bitmask.
	g_hservoD.bWhichIntsToProcess = (TW.TSRW.BYTE & TW.TIERW.BYTE);
	if (g_hservoD.bWhichIntsToProcess & 0x01) {					// GRA Match
		DSERVOTOGGLE(2);		// BUGBUG:: For debug servo stuff.
		PI_LOW((*g_pPortTable)[g_hservoD.iServoGroupStart]);	// set the servo pin low!
	}

	if (g_hservoD.bWhichIntsToProcess & 0x02) {				// GRB Match
		DSERVOTOGGLE(2);		// BUGBUG:: For debug servo stuff.
		PI_LOW((*g_pPortTable)[g_hservoD.iServoGroupStart + 1]);
	}
	
	if (g_hservoD.bWhichIntsToProcess & 0x04) {				// GRC Match
		DSERVOTOGGLE(2);		// BUGBUG:: For debug servo stuff.
		PI_LOW((*g_pPortTable)[g_hservoD.iServoGroupStart + 2]);
	}

	if (g_hservoD.bWhichIntsToProcess & 0x08) {				// GRD Match
		DSERVOTOGGLE(2);		// BUGBUG:: For debug servo stuff.
		PI_LOW((*g_pPortTable)[g_hservoD.iServoGroupStart + 3]);
	}
	// Clear out the status byte information for the bits we processed.
	TW.TSRW.BYTE  &= ~g_hservoD.bWhichIntsToProcess;
	TW.TIERW.BYTE &= ~g_hservoD.bWhichIntsToProcess;

Exit:
	DSERVOLOW(0);		// BUGBUG:: For debug servo stuff.
}
#else

#endif


//==============================================================================
// Init the ServoGroupMove data.
//==============================================================================
void HServoInit(void)			// init our stuff
{
#ifdef DEBUG_HSERVO_COMMANDS
	DBGPrintf("HServoInit\r");
#endif
#ifdef BASICATOMPRO28
#ifdef DEBUG_SERVO_PULSE		
		output(0);		// BUGBUG:: For debug servo stuff.
		output(1);		// BUGBUG:: For debug servo stuff.
		output(2);		// BUGBUG:: For debug servo stuff.
#endif		

	TW.TCNT = 0;				// give us some delay before the first interrupt...
	TW.TMRW.BYTE = 0x80;		// CTS clock no PWMD...
	TW.TSRW.BYTE = 0;			// clear out all status bytes
	TW.TIOR0.BIT.IOA = 0;		// output match - don't change any io pins
	TW.TIOR0.BIT.IOB = 0;		// output match - don't change any io pins
	TW.TIOR1.BIT.IOC = 0;		// output match - don't change any io pins
	TW.TIOR1.BIT.IOD = 0;		// output match - don't change any io pins
	
	g_hservoD.iServo = 0xff; 		// start off with a high value to start off at 0

	TW.TIERW.BIT.OVIE = 1;		// Interrupt on overflow

#else //BAP40 and arc32
	if (!g_hservoD.fInit) {
		
		g_hservoD.fInit = true;	
	
		TZ.TMDR.BIT.SYNC = 1;		// lets sync the two timer counters...
		TZ0.TCR.BYTE = 0;			// set clocks and the like.
		TZ1.TCR.BYTE = 0;			// set clocks and the like.
		TZ0.TCNT = 0;				// give us some delay before the first interrupt...
		TZ1.TCNT = 0;				// give us some delay before the first interrupt...
		TZ.TSTR.BIT.STR1 = 1;		// Make sure Timer Z1 is running 
		TZ.TSTR.BIT.STR0 = 1;		// Make sure Timer Z0 is running 
	
		TZ0.TSR.BYTE = 0;			// clear out all status bytes
		TZ1.TSR.BYTE = 0;			// clear out all status bytes
	
		TZ0.TIORA.BIT.IOA = 0;		// output match - don't change any io pins
		TZ0.TIORA.BIT.IOB = 0;		// output match - don't change any io pins
		TZ0.TIORC.BIT.IOC = 0;		// output match - don't change any io pins
		TZ0.TIORC.BIT.IOD = 0;		// output match - don't change any io pins
		
		// Initialize TZ1 for both Bap40 and Arc32
		TZ1.TIORA.BIT.IOA = 0;		// output match - don't change any io pins
		TZ1.TIORA.BIT.IOB = 0;		// output match - don't change any io pins
		TZ1.TIORC.BIT.IOC = 0;		// output match - don't change any io pins
		TZ1.TIORC.BIT.IOD = 0;		// output match - don't change any io pins
	
		int i;
		for (i=0; i< NUMTIMERCOMPAREMATCH; i++) {			// BUGBUG:: Unwind these functions into actual quick code...
			g_hservoD.abActivePins[i] = 0xff;
		}		
	
		for (i=0; i < NUMSERVOS; i++) {
			g_hservoD.ahsi[i].wPW = 0;		// see if we need to init these...
		}	
	
		g_hservoD.bChannel = 0xff;		// set it up to overflow to 0 for the first one
		g_hservoD.ulTimerOverflow = 0;	// Count of timer overflows.
	
		// if we are running on Arc32 - switch which function is handling the timerz functions...
		if (g_bWhichBap == H8_3687_ARC32) { 
			TZ.TOER.BYTE = 0;			// Enable all of the pins for timer...
	
			output(46);					// Setup the two channels to output
			output(47);
			g_hservoD.piChHigh = (*g_pPortTable)[47];			// Channel changed arc32 hi
			g_hservoD.piChLow = (*g_pPortTable)[46];			// Low...
			for (int i=48; i< 56; i++)
				input(i);				// leave the 8 outputs set to timer as input so they will
										// float when we are not having the timer drive them
	
#ifdef DEBUG_SERVO_PULSE		
			output(40);		// BUGBUG:: For debug servo stuff.
			output(41);		// BUGBUG:: For debug servo stuff.
			output(42);		// BUGBUG:: For debug servo stuff.
			output(43);		// BUGBUG:: For debug servo stuff.
#endif		
		}
#ifdef DEBUG_SERVO_PULSE		
		else {
			output(0);		// BUGBUG:: For debug servo stuff BAP 40
			output(1);		// BUGBUG:: For debug servo stuff.
			output(2);		// BUGBUG:: For debug servo stuff.
			output(3);		// BUGBUG:: For debug servo stuff.
		}
#endif	
	}

	// But make sure the overflow interrupt is enabled... On arc32 it will always be...
	g_hservoD.iServo = 0xff; 		// start off with a high value to start off at 0
	TZ0.TIER.BIT.OVIE = 1;		// Interrupt on overflow

#endif
			
	enable();					// Make sure interrupts are enabled...

}

//==============================================================================
// HServo - start up a group move...
//==============================================================================
void HServoGroupMoveStart(void)						// start a group move
{
	u8 i;

#ifdef DEBUG_HSERVO_COMMANDS
	DBGPrintf("HServoGroupMoveStart\r");
#endif	
	// Start up a group move.  If we are not already in a group move
	// initialize the group move code so we can detect which servos
	// we will process...
	if (g_hservoD.bGroupMove++ == 0) {
		for (i=0; i < NUMSERVOS; i++)
			g_hservoD.wGroupMovePW[i] = 0xffff;		// set to value we can detect
	}			
}

//==============================================================================
// HServoGroupMoveCommit - Commit any group move up a group that is in process
//==============================================================================
void HServoGroupMoveCommit(u16 wMoveTime)			// Do the commit, pass in how long you wish for the move to take
{
	u8 		i;
#ifdef DEBUG_HSERVO_COMMANDS
	DBGPrintf("HServoGroupMoveCommit %d\r", wMoveTime);
#endif
	if (wMoveTime)
		wMoveTime = (wMoveTime + 10)/20;			// convert to number of servo cycles.
	// Now do the actual group move... Detect count out of whack...
	if (g_hservoD.bGroupMove) {
		if (--g_hservoD.bGroupMove == 0) {
			// Ok it is time to commit all of the values...
			for (i=0; i < NUMSERVOS; i++) {
				if (g_hservoD.wGroupMovePW[i] != 0xffff) {
					// need to keep track of how many servos we have active 
					if (g_hservoD.wGroupMovePW[i] && !g_hservoD.ahsi[i].wPW) {
						if (++g_hservoD.cServosActive == 1)
							HServoInit();
					} else if (!g_hservoD.wGroupMovePW[i] && g_hservoD.ahsi[i].wPW) {
						g_hservoD.cServosActive--;
					}
					g_hservoD.ahsi[i].wDeltaPW = 0;								// Assume no delta, set last to minimize race conditions.
					if ((wMoveTime == 0) || (g_hservoD.wGroupMovePW[i] == 0) || (g_hservoD.ahsi[i].wPW == 0)) {
						g_hservoD.ahsi[i].wPW = g_hservoD.wGroupMovePW[i];		// set the PW to the new value
						g_hservoD.ahsi[i].wPWEnd =  g_hservoD.wGroupMovePW[i];	// Set the desired to the same
					} else {
						// First if we have not already made it to a previous timed
						// ?????? g_hservoD.ahsi[i].wPW =  g_hservoD.ahsi[i].wPWEnd; 		// move location, get there now...
						g_hservoD.ahsi[i].wPWEnd =  g_hservoD.wGroupMovePW[i];	// Set the desired to the new time
						
						// calculate the delta per cycle..
						if (g_hservoD.ahsi[i].wPWEnd != g_hservoD.ahsi[i].wPW) {
							g_hservoD.ahsi[i].wDeltaPW = imax((iabs(g_hservoD.ahsi[i].wPWEnd - 
									g_hservoD.ahsi[i].wPW) + wMoveTime/2) / wMoveTime, 1);	// make sure it moves...
							
						}
					}					
				}
			}
		}
	}	
#if 0	
	for (i=0; i < NUMSERVOS; i++) {
		DBGPrintf("%d - %x %x %x\r", i, g_hservoD.ahsi[i].wPW, g_hservoD.ahsi[i].wPWEnd, g_hservoD.ahsi[i].wDeltaPW);
	}
#endif	
}


//==============================================================================
// HServo - start up a group move...
//==============================================================================
void HServo(u8 pin, u16 wPulseWidth)
{
#ifdef DEBUG_HSERVO_COMMANDS
	DBGPrintf("HServo %d %d\r", pin, wPulseWidth);
#endif
	if (pin >= NUMSERVOS)
		return;	// error out if pin is out of range...

	// First convert the pulse width to clocks for the processor and add in the indivisual offset
	if (wPulseWidth) {
		wPulseWidth = (wPulseWidth + g_hservoD.asServoOffsets[pin]) * clockCyclesPerMicrosecond ;			// Clocks per millisecond.
	}	
	
	if (g_hservoD.ahsi[pin].wPW == 0)
		output(pin);			// only change to output if this is the first move
								// As on Arc32 we change from input to output depending
								// on if the servo is in the group being processed
	// See if we are in a group move...
	if (g_hservoD.bGroupMove) {
		g_hservoD.wGroupMovePW[pin] = wPulseWidth;
	} else {
		// need to keep track of how many servos we have active 
		if (wPulseWidth && !g_hservoD.ahsi[pin].wPW) {
			if (++g_hservoD.cServosActive == 1)
				HServoInit();
		} else if (!wPulseWidth && g_hservoD.ahsi[pin].wPW) {
			g_hservoD.cServosActive--;
		}
		
		g_hservoD.ahsi[pin].wDeltaPW = 0;			// No delta time stuff
		g_hservoD.ahsi[pin].wPW = wPulseWidth;		// set the PW to the new value
		g_hservoD.ahsi[pin].wPWEnd =  wPulseWidth;	// Set the desired to the same
	}
}


//==============================================================================
// HServoIdle - Return true if the servo is idle - that is it at it's end point
// 	location (ie not still part of a group move)
//==============================================================================
bool HServoIdle(u8 pin)
{
	return (g_hservoD.ahsi[pin].wPW == g_hservoD.ahsi[pin].wPWEnd);
}


//==============================================================================
// HServoPos - Return the current position for a servo
//==============================================================================
extern u16 HServoPos(u8 pin)
{
	// Need to handle our offsets.
	register u16 wT;
	wT = g_hservoD.ahsi[pin].wPW;
	if (!wT)	// don't offset if we have no pulse...
		return 0;
	return (wT / clockCyclesPerMicrosecond) -  g_hservoD.asServoOffsets[pin];
}

//==============================================================================
// HServoWait - Wait for all of the servos specified in the mask to be at their
//			final position...
//==============================================================================
void HServoWait(u32 lServoMask)						// Have a wait function we pass in mask of which servos we wish to wait for...
{
	for(u8 iPin=0; lServoMask != 0; iPin++) {
		if (lServoMask & 1) {	// this servo is part of the group move
			while (g_hservoD.ahsi[iPin].wPW != g_hservoD.ahsi[iPin].wPWEnd)
				; //wait until this one gets to it's end point
		}
		lServoMask >>= 1;
	}		
}


//==============================================================================
// HServoSetServoOffset - Set the offset for a specific servo.  Currently in
// 			microseconds, but may choose to go with CPU cycles...
//==============================================================================
void HServoSetServoOffset(u8 pin, short sOffset)	// Set an offset for a servo
{
	if (pin < NUMSERVOS)
		g_hservoD.asServoOffsets[pin] = sOffset;
}

//==============================================================================
// HServoWait - Wait for all of the servos specified in the mask to be at their
//			final position...
//==============================================================================
short HServoGetServoOffset(u8 pin)
{
	
	if (pin < NUMSERVOS)
		return g_hservoD.asServoOffsets[pin];
	return 0;	// error? 
}
