//==============================================================================
// Servo_Arc32.cpp - This file contains servo support that hopefully should be similiar
//		to the hservo2 system used in basic.  Will first start off just getting
//		servos to work, will then add in the timed group moves.
//  Not sure if unit base.  g_hservoD.iIndex am thinking in uSs as that is what most people
//	are used to...  Internally we will convert everything to clock tick units...
//
// Version 0.1 - Work in Progress
//
// Major Contributers: Kurt Eckhardt 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================

#include "baplib.h"
#include "Servo.h"
#include "servo_int.h"

#ifdef H8_FAMILY_3687
extern volatile unsigned int* apwGRx[];

// This is the HSERVO implementation.
//=============================================================================
// TIMERZ0() - BAP40 version 
// Main timerw functino that is used to handle the background processing
// of the servo sub-system
//=============================================================================
extern "C" void TIMERZ0_BAP40() __attribute__ ((interrupt_handler));
extern "C" void TIMERZ0_BAP40()
{
	// See why we woke up for
	DSERVOHIGH(0);		// BUGBUG:: For debug servo stuff.
	if (TZ0.TSR.BIT.OVF) {
		DSERVOTOGGLE(1);		// BUGBUG:: For debug servo stuff.
#define SERVO_TIMING_FUDGE 70

		// If no servos are currently active we can now bail as we have
		// set all of the previously active servos back to low...
		if (!g_hservoD.cServosActive) {
			TZ0.TSR.BYTE = 0;		// turn off all of the status bytes.
			TZ0.TIER.BYTE = 0;		// Turn off all interrupts
			TZ1.TSR.BYTE = 0;		// turn off all of the status bytes.
			TZ1.TIER.BYTE = 0;		// Turn off all interrupts
			goto Exit;
		}

		g_hservoD.bChannel++;		// increment the channel.
		if (g_hservoD.bChannel > 5) // handle wrap
			g_hservoD.bChannel = 0;

		if (g_hservoD.bChannel <= 3) {

			TZ1.TIER.BYTE = 0;				// Turn off all interrupts
			g_hservoD.bMask = 1;			// Save from doing counted shifts...

			if (g_hservoD.iServo >= NUMSERVOS) {
				g_hservoD.iServo = 0;
			}
			g_hservoD.iServoGroupStart = g_hservoD.iServo;

			// Process in a loop Makes shorter number of lines, but probably a bit slower
			for (g_hservoD.iIndex=0; g_hservoD.iIndex < 8; g_hservoD.iIndex++) {
				if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {

					// First see if we need to adjust the servo position
					IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
		
					// servo is active so lets get all of the timers and the like setup
					PI_HIGH((*g_pPortTable)[g_hservoD.iServo]);	// So set this servo pin high now...

					// Use Table to set the correct interrupt
					if (g_hservoD.iIndex < 4) {
						*(apwGRx[g_hservoD.iIndex]) = TZ0.TCNT + g_hservoD.ahsi[g_hservoD.iServo].wPW - SERVO_TIMING_FUDGE; 
						TZ0.TIER.BYTE |= g_hservoD.bMask;		// enable that interrupt
					} else {
						*(apwGRx[g_hservoD.iIndex]) = TZ1.TCNT + g_hservoD.ahsi[g_hservoD.iServo].wPW - SERVO_TIMING_FUDGE; 
						TZ1.TIER.BYTE |= g_hservoD.bMask;		// enable that interrupt
					}
				}
				g_hservoD.bMask <<= 1;
				if (g_hservoD.bMask == (1<<4))			// Handle switch to TZ1...
					g_hservoD.bMask = 1;

				g_hservoD.iServo++; // setup to process the next servo...
			}
		}
		
		TZ0.TSR.BYTE = 0x0;								// Clear all of the status bits
		TZ1.TSR.BYTE &= 0xf0;								// Clear all of the status bits
		goto Exit;
	}

	// Lets see which of the 4 channels we woke up for.  Lets try with pre-generated bitmask.
	g_hservoD.bWhichIntsToProcess = (TZ0.TSR.BYTE & TZ0.TIER.BYTE);
	if (g_hservoD.bWhichIntsToProcess & 0x01) {					// GRA Match
		DSERVOTOGGLE(2);		// BUGBUG:: For debug servo stuff.
		PI_LOW((*g_pPortTable)[g_hservoD.iServoGroupStart]);	// set the servo pin low!
	}

	if (g_hservoD.bWhichIntsToProcess & 0x02) {				// GRB Match
		DSERVOTOGGLE(2);		// BUGBUG:: For debug servo stuff.
		PI_LOW((*g_pPortTable)[g_hservoD.iServoGroupStart + 1]);
	}
	
	if (g_hservoD.bWhichIntsToProcess & 0x04) {				// GRC Match
		DSERVOTOGGLE(2);		// BUGBUG:: For debug servo stuff.
		PI_LOW((*g_pPortTable)[g_hservoD.iServoGroupStart + 2]);
	}

	if (g_hservoD.bWhichIntsToProcess & 0x08) {				// GRD Match
		DSERVOTOGGLE(2);		// BUGBUG:: For debug servo stuff.
		PI_LOW((*g_pPortTable)[g_hservoD.iServoGroupStart + 3]);
	}
	// Clear out the status byte information for the bits we processed.
	TZ0.TSR.BYTE  &= ~g_hservoD.bWhichIntsToProcess;
	TZ0.TIER.BYTE &= ~g_hservoD.bWhichIntsToProcess;

Exit:
	DSERVOLOW(0);		// BUGBUG:: For debug servo stuff.

}

//=============================================================================
// TIMERZ1_BAP40 - Timer z1 for Bap40...
// the GRA-GRD interrupt as timerz0 will do all of the setup for overflow...
//=============================================================================

extern "C" void TIMERZ1_BAP40() __attribute__ ((interrupt_handler));
extern "C" void TIMERZ1_BAP40()
{

	g_hservoD.bWhichIntsToProcess = TZ1.TSR.BYTE & TZ1.TIER.BYTE;	// Only process the bits that are enabled...

	// Bap40 version
	// Lets see which of the 4 channels we woke up for.  Lets try with pre-generated bitmask.
	DSERVOTOGGLE(3);		// BUGBUG:: For debug servo stuff.

	if (g_hservoD.bWhichIntsToProcess & 0x01) {					// GRA Match
		DSERVOTOGGLE(2);		// BUGBUG:: For debug servo stuff.
		PI_LOW((*g_pPortTable)[g_hservoD.iServoGroupStart + 4]);	// set the servo pin low!
	}

	if (g_hservoD.bWhichIntsToProcess & 0x02) {				// GRB Match
		DSERVOTOGGLE(2);		// BUGBUG:: For debug servo stuff.
		PI_LOW((*g_pPortTable)[g_hservoD.iServoGroupStart + 5]);
	}
	
	if (g_hservoD.bWhichIntsToProcess & 0x04) {				// GRC Match
		DSERVOTOGGLE(2);		// BUGBUG:: For debug servo stuff.
		PI_LOW((*g_pPortTable)[g_hservoD.iServoGroupStart + 6]);
	}

	if (g_hservoD.bWhichIntsToProcess & 0x08) {				// GRD Match
		DSERVOTOGGLE(2);		// BUGBUG:: For debug servo stuff.
		PI_LOW((*g_pPortTable)[g_hservoD.iServoGroupStart + 7]);
	}
	
	TZ1.TIER.BYTE &= ~g_hservoD.bWhichIntsToProcess;
	TZ1.TSR.BYTE &= ~g_hservoD.bWhichIntsToProcess;
	
}

#endif // H8_FAMILY_3687
