//==============================================================================
// SERVO_INT.H - Internal header for servo code...
// Version 0.1 - Work in Progress - Warning this work has not been debugged yet!!!
//
// Major Contributers: Kurt Eckhardt 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================
#ifndef Servo_Int_h
#define Servo_Int_h
//#define DEBUG_HSERVO_COMMANDS
//#define DEBUG_SERVO_PULSE

#ifdef DEBUG_SERVO_PULSE
#define DSERVOTOGGLE(_pn_) toggle(_pn_)
#define DSERVOHIGH(_pn_) high(_pn_)
#define DSERVOLOW(_pn_) low(_pn_)
#else
#define DSERVOTOGGLE(_pn_) 
#define DSERVOHIGH(_pn_)
#define DSERVOLOW(_pn_)
#endif


// For each servo we will store
// 
#ifdef BASICATOMPRO28
#define NUMSERVOS				20	// We are set up for a maximum of 20 servos
#define NUMTIMERCOMPAREMATCH	4	// Number of timer compare match units we have
#else
#define NUMSERVOS				32	// We are set up for a maximum of 32 servos on Bap40/Arc32
#define NUMTIMERCOMPAREMATCH	8	// Number of timer compare match units we have
#endif

typedef struct _hsitem
{
	u16		wPW;			// Current pulse width in clocks
	u16		wPWEnd;			// Desired pulse width after a group move
	u16		wDeltaPW;		// how much to change per cycle
} HSITEM;

typedef struct _hsevero_data
{
	HSITEM	ahsi[NUMSERVOS];
	u16		wGroupMovePW[NUMSERVOS];	// place to remember new values for a group move.
	short	asServoOffsets[NUMSERVOS];	// offsets to use for the servos
	u8		fInit;						// Have we called HservoInit before?
	u8		cServosActive;		// Are there any servos active?
	u8 		bGroupMove;			// are we in a group move? - keep a counter so can nest...
	u8		iServo;				// which servo to start with for this pass
	u8		iServoGroupStart;	// Which servo number did this group start with?
	u8		wFudge;				// fudge factor to get closer to exact pulses
	u8		bWhichIntsToProcess; // combination of which are enabled and which have happened.

#ifdef H8_FAMILY_3687
	u8		abActivePins[NUMTIMERCOMPAREMATCH];	// know which pins are active. = 0xff if not active...
	u8		bChannel;			// which channel of servos are we processing.
	u32		ulTimerOverflow;	// Count of timer overflows.
	u8		bMask;				// Mask to use to walk through interrupts.
	u8		iIndex;				// temporary index to use.
	PORTITEM *ppiTemp;			// Temporary port item 
	PORTITEM piChHigh;			// Channel changed arc32 hi
	PORTITEM piChLow;			// Low...
#endif
} HSERVO_DATA;

extern HSERVO_DATA g_hservoD;

#define usToTicks(_us)    (unsigned int)(((unsigned long) clockCyclesPerMicrosecond * _us) / 8)     // converts microseconds to tick (assumes prescale of 8)  
#define ticksToUs(_ticks) (( (unsigned long)_ticks * 8)/ (unsigned long)clockCyclesPerMicrosecond ) // converts from ticks back to microseconds

#define TRIM_DURATION       2                               // compensation ticks to trim adjust for digitalWrite delays // 12 August 2009


//=============================================================================
// Inline function that is called to update each servos position in handling
// a timed servo move
//=============================================================================
__inline__ void IncrementalUpdateServoPos(HSITEM* phsi) __attribute__ ((always_inline));
__inline__ void IncrementalUpdateServoPos(HSITEM* phsi)
{
	// Also see if we need to adjust the pulse width for the next time...
	if (phsi->wDeltaPW) {
		if (phsi->wPW < phsi->wPWEnd) {
			phsi->wPW += phsi->wDeltaPW;
			if (phsi->wPW >= phsi->wPWEnd){ // reached the end point
				phsi->wPW = phsi->wPWEnd;
				phsi->wDeltaPW = 0;
			}
		} else {
			phsi->wPW -= phsi->wDeltaPW;
			if (phsi->wPW <= phsi->wPWEnd){ // reached the end point
				phsi->wPW = phsi->wPWEnd;
				phsi->wDeltaPW = 0;
			}
		}
	}
}

#endif
