//==============================================================================
/*
  Striped down version for Baplib Arduino like code on Basic Atom Pros
  Part of Arduino - http://www.arduino.cc/

  Copyright (c) 2005-2006 David A. Mellis

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA

  $Id: wiring.h 804 2009-12-18 16:05:52Z dmellis $
  Changed by: KurtE
*/

#ifndef Wiring_h
#define Wiring_h

#ifdef __cplusplus
extern "C"{
#endif

//#define HIGH 0x1
//#define LOW  0x0

//#define INPUT 0x0
//#define OUTPUT 0x1

#define true 0x1
#define false 0x0

#define PI 3.1415926535897932384626433832795
#define HALF_PI 1.5707963267948966192313216916398
#define TWO_PI 6.283185307179586476925286766559
#define DEG_TO_RAD 0.017453292519943295769236907684886
#define RAD_TO_DEG 57.295779513082320876798154814105

#define SERIAL  0x0
#define DISPLAY 0x1

#define LSBFIRST 0
#define MSBFIRST 1

#define CHANGE 1
#define FALLING 2
#define RISING 3

#define INTERNAL 3
#define DEFAULT 1
#define EXTERNAL 0

// undefine stdlib's abs if encountered
#ifdef abs
#undef abs
#endif

#define min(a,b) ((a)<(b)?(a):(b))
#define max(a,b) ((a)>(b)?(a):(b))
#define abs(x) ((x)>0?(x):-(x))
#define constrain(amt,low,high) ((amt)<(low)?(low):((amt)>(high)?(high):(amt)))
#define round(x)     ((x)>=0?(long)((x)+0.5):(long)((x)-0.5))
#define radians(deg) ((deg)*DEG_TO_RAD)
#define degrees(rad) ((rad)*RAD_TO_DEG)
#define sq(x) ((x)*(x))

//#define interrupts() sei()
//#define noInterrupts() cli()

//#define clockCyclesPerMicrosecond() ( F_CPU / 1000000L )
//#define clockCyclesToMicroseconds(a) ( (a) / clockCyclesPerMicrosecond() )
//#define microsecondsToClockCycles(a) ( (a) * clockCyclesPerMicrosecond() )

#define lowByte(w) ((unsigned char) ((w) & 0xff))
#define highByte(w) ((unsigned char) ((w) >> 8))

#define bitRead(value, bit) (((value) >> (bit)) & 0x01)
#define bitSet(value, bit) ((value) |= (1UL << (bit)))
#define bitClear(value, bit) ((value) &= ~(1UL << (bit)))
#define bitWrite(value, bit, bitvalue) (bitvalue ? bitSet(value, bit) : bitClear(value, bit))

//typedef unsigned int word;

#define bit(b) (1UL << (b))

typedef unsigned char boolean;
//typedef unsigned char byte;

void init(void);

void pinMode(unsigned char, unsigned char);
void digitalWrite(unsigned char, unsigned char);
int digitalRead(unsigned char);
int analogRead(unsigned char);
void analogReference(unsigned char mode);
void analogWrite(unsigned char, int);

void beginSerial(long);
void serialWrite(unsigned char);
int serialAvailable(void);
int serialRead(void);
void serialFlush(void);

unsigned long millis(void);
unsigned long micros(void);
void delay(unsigned long);
void delayMicroseconds(unsigned int us);
unsigned long pulseIn(unsigned char pin, unsigned char state, unsigned long timeout);

// Provide a couple of wrapper functions to emulate Arduino functinos
extern unsigned long micros(void);
extern unsigned long millis(void);
extern long map(long x, long in_min, long in_max, long out_min, long out_max);
void shiftOut(unsigned char dataPin, unsigned char clockPin, unsigned char bitOrder, byte val);

void attachInterrupt(unsigned char, void (*)(void), int mode);
void detachInterrupt(unsigned char);

void setup(void);
void loop(void);

#ifdef __cplusplus
} // extern "C"
#endif

#endif
