//******************************************************************************
// Kurt's Brat - in C
//
//*****************************************************************************
#include "globals.h"
#ifdef USEXBEE
#include "..\BAPLib\diyxbee.h"
#endif
#include "..\baplib\ps2sup.h"
#include "..\baplib\servo.h"
#include "..\baplib\eeprom.h"

//-----------------------------------------------------------------------------
// Define our IO pins...

#define NUMSERVOS	7
#define LAPIN		P4 
#define LKPIN		P5 
#define LHPIN		P6 
#define RAPIN		P7 
#define RKPIN		P8
//;Speaker 			p
#define RHPIN		P10
#define TTPIN		P11

const u8	c_abServoTbl[] = {RHPIN, RKPIN, RAPIN, LHPIN, LKPIN, LAPIN, TTPIN};




#define DEBUG

//user variables
#ifdef USEXBEE
DIYPACKET 			g_diypPrev;
DIYPACKET 			g_diyp;				// We are assuming that we will keep the previous one from run to run...
#endif
PS2PACKET 			g_ps2p;
PS2PACKET 			g_ps2pPrev;

unsigned long 		g_ulLastActive;		// timer of when we were last active
bool 				g_fCommandActive;
unsigned long 		g_ulPauseStart;		// time when we started a pause
unsigned long 		g_ulPausePeriod;	// how much time to pause.

unsigned long 		g_ulInputPauseStart;	// time when we started a pause
unsigned long 		g_ulInputPausePeriod;	// how much time to pause.

unsigned long		g_ulLastDebug;		

#define ANGLE1_TO_SERVO_PW(ang1)	((((long)(ang1) * 1666) / 1600)+1500)

// Now lets define a simple Move data structure to make things easy for us
typedef struct _mlsd1 {
	short rightanklepos;
	short rightkneepos;
	short righthippos;
	short leftanklepos;
	short leftkneepos;
	short lefthippos;
	short sMoveTime;
} MLSD1;
typedef struct _mlsd1 *PMLSD1;

// This holds multiple moves in a counted structure
typedef struct _cmlsd1 {
	short cItems;
	MLSD1 amlsd1[10];
} CMLSD1, *PCMLSD1;


#define MNUMSERVOS 6
typedef struct _brat_save_data
{
	u8	cServos;		// count of servos in our save data.
	u8	bChksum;		// Checksum.  simple check to see if the data is valid
	struct _asd				// Servo Data
	{
		u16	center_us;	// The number of microseconds pulse length to center the servo. On ideal servos it will be 1500. ie 1,5ms
		u16	range_us;	// The number of microseconds either side of center for full range. On ideal servos it be 500. Range = 1ms to 2ms.
	} asd[MNUMSERVOS];

} BRAT_SAVE_DATA;

long 				g_lWalkSpeed1;
long 				g_lWalkAngle1;
;bool 			fAbortCommand;
short				g_sLastStep;
short				g_sAnkleAdj;
short				g_sTravLength;
bool 			g_fBotActive;
bool				g_fPS2Input;
bool 			g_fFindServoZeros;

#ifdef DEBUG
bool g_fDebugBrat;
#endif
#define TravelDeadZone 5
#define TRUE -1
#define FALSE 0

// define some forward references
extern short GetIRScanInCM(u8 bPin);
//extern void FindServoZeroPoints(SERVO_LIST *pservos);
extern void MoveLegServosDegrees1 (short rightanklepos,short rightkneepos,short righthippos,short leftanklepos,short leftkneepos,short lefthippos,short sMoveTime);
extern bool FProcessCurrentCommandList(void);
extern void	SetCurrentCommandList(const PCMLSD1 pcmlsd1);	
extern void ReadServoOffsets(void);

//#define PS2_TEST  1
#ifdef PS2_TEST
extern void PS2_Test(void);
#endif
#ifdef USEXBEE
extern unsigned long CheckAndProcessDIYXBeeData(void);
#endif

extern unsigned long CheckAndProcessPS2Data(void);
//extern void LoadServosConfig(SERVO_LIST *pservos, short cServos);		// OK 
//#define SERVO_TEST
#ifdef SERVO_TEST
extern void ServoRangeTest(void);
#endif

extern void InitController(void);
extern bool FLegServosStillMoving(void);
 
PCMLSD1 g_pcmlsd1;				// Current command list that is active
short	g_imlsd1;				// Current index to which command in the command list is active.
#define SET_MLSD1(pm,ra,rk,rh,la,lk,lh, time)  \
	(pm)->rightanklepos = (ra);	\
	(pm)->rightkneepos = (rk);	\
	(pm)->righthippos = (rh);		\
	(pm)->leftanklepos = (la);	\
	(pm)->leftkneepos = (lk); 	\
	(pm)->lefthippos = (lh);		\
	(pm)->sMoveTime = (time);
	
CMLSD1 g_cmlsd1_Walk = {2, {{0,0,0,0,0,0,500}, {0,0,0,0,0,0,500}}};

#ifdef LATER
const CMLSD1 s_cmlsd1_KickLeft = {6, {
         {  0,   0,  0,  0,   0,   0, 500}, 
         {-140,  0,  0, 420,  0,   0, 500}, 
         {-230,  0,  0,  0,-320, 410, 500}, 
         {-230,  0,  0,  0, 240,-200, 250}, 
         {-180,  0,  0,  0,   0,   0, 500}, 
         {   0,  0,  0,  0,   0,   0, 500}}};

const CMLSD1 s_cmlsd1_KickRight = {6, {
	     {   0,   0,   0,  0,   0,  0, 500},
         { 420,   0,   0,-140,  0,  0, 500}, 
         {   0,-320, 410,-230,  0,  0, 500}, 
         {   0, 240,-200,-230,  0,  0, 250}, 
         {   0,   0,   0,-180,  0,  0, 500}, 
         {   0,   0,   0,   0,  0,  0, 500}}}; 
#endif
const CMLSD1 s_cmlsd1_UpFromFront = {6, {
         {   0,   0,   0,  0,   0,   0, 500}, 
         {   0, 900, 450,  0, 900, 450, 500}, 
         { 400, 900,-370,  0, 900, 450, 500}, 
         {   0, 900,-650,  0, 900,-650, 500}, 
         {   0, 900,-650,  0, 900,-650, -300}, // Was a pause 300 may put in something special for this...
		 {  0,   0,   0,   0,   0,   0,1000}}}; 

const CMLSD1 s_cmlsd1_UpFromBack = {3, {
         {  0,   0,   0,  0,   0,  0,1000},
         {  0,-900,  50, 00,-900,  50,1000}, 
         {  0,   0,   0,  0,   0,  0,1000}}};


const CMLSD1 s_cmlsd1_Home = {1, {
     	{  0,  0,  0,  0,  0,  0, 500} }};

const CMLSD1 s_cmlsd1_TurnLeft = {9, {
         { 200,   0,   0,-140,   0,   0,500}, 
         { 200,-350,-350,-140, 350, 350,500}, 
         {   0,-350,-350,   0, 350, 350,500}, 
         {   0, 350, 350,   0,-350,-350,500}, 
         { 200, 350, 350,-140,-350,-350,500}, 
         { 200,   0,   0,-140,-350,-350,500},
         {-140,   0,   0, 200,-350,-350,500},
         {-140,   0,   0, 200,   0,   0,500},
         {   0,   0,   0,   0,   0,   0,500}}};

const CMLSD1 s_cmlsd1_TurnRight = {9, {
         {-140,   0,   0, 200,  0,   0,500} ,
         {-140, 350, 350, 200,-350,-350,500}, 
         {   0, 350, 350,   0,-350,-350,500}, 
         {   0,-350,-350,   0, 350, 350,500}, 
         {-140,-350,-350, 200, 350, 350,500}, 
         {-140,-350,-350, 200,   0,   0,500},
         { 200,-350,-350,-140,   0,   0,500},
         { 200,   0,   0,-140,   0,   0,500},
         {   0,   0,   0,   0,   0,   0,500}}};

const CMLSD1 s_cmlsd1_CCLeft = {6, {
         {   0,   0,   0,   0,   0,   0,500},
         { 250,   0,   0,-180,   0,   0,500},
         { 250,   0,   0,-180,   0,   0,-100},		// was a delay of 100;
         {  20,-200,-200,-180, 200, 200,500},
         {   0,-200,-200,   0, 200, 200,500},
         {   0,   0,   0,   0,   0,   0,500}}};

const CMLSD1 s_cmlsd1_CCRight = {6, {
         {   0,   0,   0,   0,   0,   0,500},
         {-180,   0,   0, 250,   0,   0,500},
         {-180,   0,   0, 250,   0,   0,-100},	// was a delay of 100
         {-180, 200, 200,  20,-200,-200,500},
         {   0, 200, 200,   0,-200,-200,500},
         {   0,   0,   0,   0,   0,   0,500}}};

const CMLSD1 s_cmlsd1_Rest = {1, {
         {   0, 350, 400,   0, 350, 400,500}}};

#ifdef USEXBEE
static u8 s_abSND_RobotOff[] 	= {100, 4400/25, 80, 3800/25, 60, 3200/25};
static u8 s_abSND_RobotOn[] 	= {60, 3200/25, 80,  3800/25, 100, 4400/25};
#endif
u8	bICIER;		// BUGBUG try to figure out who is screwing this under...
u8	bICSR;

//=============================================================================
// Main Function
//=============================================================================
int main(void)
{
	//[INIT]
	bap_initialize();

	bICIER = IIC2.ICIER.BYTE;
	bICSR = IIC2.ICSR.BYTE;
	
	// Tell ssytem that we would like to trace faults
	g_fDBGHandleError = true;
	
	low(9);	// Sound
	output(9);	

	//Timer
	InitSystemTimer();			// Initialize the system timer
	enable();					//enables all interrupts


	//Initialize Controller
	InitController();


	// Init the servos
	ReadServoOffsets();

	
	// Make sure everything is init OK
	g_pcmlsd1 = 0;		// Current command list that is active
	g_imlsd1 = 0;		// Current index to which command in the command list is active.
	g_ulPausePeriod = 0; // make sure we don't start off with some delay defined
	g_ulLastActive = GetCurrentTime();	// remember our time
#ifdef DEBUG
	g_fDebugBrat = TRUE;			// start with it off!
#endif	
	g_fPS2Input = FALSE;				// default to PS2 Input
	g_sTravLength = 5;				// default for ps2...

	// Now go to the main loop;

#ifdef PS2_TEST
		PS2_Test();
#endif

	// Lets try to init all of the servos to a zero point...
//	MoveLegServosDegrees1 (0,0,0,0,0,0, 0);

	g_ulInputPausePeriod = 0;	// make sure we get the first call to the input driver happens...


	//-------------------------------------------------------------------------
	// Main loop
	//-------------------------------------------------------------------------
	while (1) {
		// call off to process our current command list (if any).  It will return TRUE if a command is still active
		// this will influence which commands we may process while a command is still active.

		g_fCommandActive = FProcessCurrentCommandList();	
	
		if (FLegServosStillMoving())					// check to see if any group moves are going on...
			g_ulLastActive = GetCurrentTime();			// remember our time


		// We wish maybe wait between calls to the input.  For example PS2 does not work if we call too often...		
		if (!g_ulInputPausePeriod || FTimeHasElapsed(g_ulInputPauseStart, g_ulInputPausePeriod)) {
#ifdef USEXBEE	
			if (g_fPS2Input)
#endif
				g_ulInputPausePeriod = CheckAndProcessPS2Data();
#ifdef USEXBEE			
			else
				g_ulInputPausePeriod = CheckAndProcessDIYXBeeData();
#endif

			g_ulInputPauseStart = GetCurrentTime();
		}
	}
}	

//==============================================================================
// InitController: do what is ncessary to initialize the controller(s)
//==============================================================================
void InitController()
{
	// Init the PS2...
	PS2_Init(PS2_DAT, PS2_CMD, PS2_SEL, PS2_CLK);
#ifdef USEXBEE
	// Lets try to initialize the XBEE to use
	InitXBee(_S_HSERIAL_, _S_HSERIAL_, P7, 62500);

	
	// Clear any stuff left in the buffer
	pause(20);
	ClearXBeeInputBuffer();
#endif
}

//==============================================================================
// ServoTest - Quick and dirty sweep test of one servo
//==============================================================================
void ServoTest()
{
	MSound(9, 2, 80, 2200, 80, 2200);

	HServoGroupMoveStart();
	HServo(11, 1500);
	HServoGroupMoveCommit(200);		// Center us to start...
	while (!HServoIdle(11))
		;

	HServoGroupMoveStart();
	HServo(11, 2250);
	HServoGroupMoveCommit(1000);		// Go one direction
	while (!HServoIdle(11))
		;

	HServoGroupMoveStart();
	HServo(11, 750);
	HServoGroupMoveCommit(2000);		// Go the other
	while (!HServoIdle(11))
		;

	HServoGroupMoveStart();
	HServo(11, 2250);
	HServoGroupMoveCommit(2000);		// Go the other
	while (!HServoIdle(11))
		;	

	HServoGroupMoveStart();
	HServo(11, 1500);
	HServoGroupMoveCommit(1000);		// return to center
	while (!HServoIdle(11))
		;

	MSound(9, 2, 80, 2000, 80, 2000);
}

//==============================================================================
// This is code the checks for and processes input from the DIY XBee receiver
//			work
//==============================================================================
unsigned long CheckAndProcessPS2Data(void)
{
	bool fPacketChanged;
	short	JoyStickValueIn;
	short int sT;
	
	// Then try to receive a packet of information from the PS2.
//	DBGPrintf("CheckAndProcessPs2Data\r");
	output(PS2_CMD);
	output(PS2_SEL);
	output(PS2_CLK);
	input(PS2_DAT,true);
	if (PS2_Query(g_ps2p.bData, sizeof(g_ps2p))) {
		fPacketChanged = !FCompareMem((const char *)g_ps2p.bData, (const char *)g_ps2pPrev.bData, sizeof(g_ps2p));

		if (fPacketChanged) {
#ifdef DEBUG
			// setup to output back to our USB port
			if (g_fDebugBrat) 	{
//				DBGPrintf("%2x%2x %2x %2x %2x %2x\n\r",g_ps2p.bData[0],g_ps2p.bData[1], g_ps2p.s.bRJoyLR, g_ps2p.s.bRJoyLR, 
//						g_ps2p.s.bLJoyLR, g_ps2p.s.bLJoyUD);
			}
#endif
		}

		// OK lets try "0" button for Start. 
		if (!g_ps2p.s.fNStart && g_ps2pPrev.s.fNStart)	{	//Start Button (0 on keypad) test
			if(g_fBotActive) {
				//Turn off
				//Sound P9,[100\4400,80\3800,60\3200];
      			MSound(9, 3, 100, 2200, 80, 1900, 60, 1600);

				SetCurrentCommandList((PCMLSD1)&s_cmlsd1_Rest);		//command = 9;
				g_fBotActive = FALSE; 
			} else {
				//Turn on
				//Sound P9,[60\3200,80\3800,100\4400];
				MSound(9, 3, 60, 1600, 80, 1900, 100, 2200);
				SetCurrentCommandList((PCMLSD1)&s_cmlsd1_Home);		//command = 0
				g_fBotActive = TRUE;
#ifdef DEBUG
				if (g_fDebugBrat) 	{
					DBGPrintf("Offsets: %d %d %d : %d %d %d\n\r", HServoGetServoOffset(RHPIN), HServoGetServoOffset(RKPIN), HServoGetServoOffset(RAPIN),
							HServoGetServoOffset(LHPIN), HServoGetServoOffset(LKPIN), HServoGetServoOffset(LAPIN));
				}
#endif
			}
		}	

		// Use R3 button to run a servo test...		
		if (!g_ps2p.s.fNR3 && g_ps2pPrev.s.fNR3) {	//R3 button.  Lets try servo test
			ServoTest();
		}


		if (g_fBotActive) {
			if (!g_ps2p.s.fNUp && g_ps2pPrev.s.fNUp)					// UP button on PS2 button test
				SetCurrentCommandList((PCMLSD1)&s_cmlsd1_UpFromFront);		//command = 7

			else if (!g_ps2p.s.fNDown && g_ps2pPrev.s.fNDown)			// Down
				SetCurrentCommandList((PCMLSD1)&s_cmlsd1_UpFromBack);		//command = 8
		
			else if (!g_ps2p.s.fNX && g_ps2pPrev.s.fNX)				// X button test
				SetCurrentCommandList((PCMLSD1)&s_cmlsd1_Home);				//command = 0
	
			else if (!g_ps2p.s.fNSelect && g_ps2pPrev.s.fNSelect)	{	// Select button test
				// Can adjust the travel length...
				if (g_sTravLength == 5)
					g_sTravLength = 8;
				else
					g_sTravLength = 5;

			}
			// Commands that are set by joystick should only be set when a command is not currently active.	
			else if (!g_fCommandActive) {
				if (iabs((short)g_ps2p.s.bRJoyUD-128) > TravelDeadZone) {
				 
					// Note, Speed and Angle here are in 10s so multiply by 10...		
					JoyStickValueIn = (short)g_ps2p.s.bRJoyUD - 128;
					if (JoyStickValueIn > 0) {
						g_lWalkSpeed1 = (JoyStickValueIn*10)/g_sTravLength;
						g_lWalkAngle1 = (JoyStickValueIn + 40) - g_sAnkleAdj;
					} else {
						g_lWalkSpeed1 = (JoyStickValueIn*10)/g_sTravLength;
						g_lWalkAngle1 = -((JoyStickValueIn - 40) + g_sAnkleAdj);
					}

					// We are setting up to do our main walk forward or backward.

					if (g_sLastStep == 0) {
						SET_MLSD1(&g_cmlsd1_Walk.amlsd1[0],g_lWalkAngle1, g_lWalkSpeed1, g_lWalkSpeed1, -g_lWalkAngle1,-g_lWalkSpeed1,-g_lWalkSpeed1,500);
						SET_MLSD1(&g_cmlsd1_Walk.amlsd1[1],-g_lWalkAngle1, g_lWalkSpeed1, g_lWalkSpeed1,  g_lWalkAngle1,-g_lWalkSpeed1,-g_lWalkSpeed1,500);
						g_sLastStep = 1;
					} else if (g_sLastStep == 1) {
						SET_MLSD1(&g_cmlsd1_Walk.amlsd1[0], -g_lWalkAngle1,-g_lWalkSpeed1,-g_lWalkSpeed1,  g_lWalkAngle1, g_lWalkSpeed1, g_lWalkSpeed1,500); 
						SET_MLSD1(&g_cmlsd1_Walk.amlsd1[1], g_lWalkAngle1,-g_lWalkSpeed1,-g_lWalkSpeed1, -g_lWalkAngle1, g_lWalkSpeed1, g_lWalkSpeed1,500); 
						g_sLastStep = 0;
					}
					g_sAnkleAdj = g_sAnkleAdj + 10;

					if (g_sAnkleAdj > 60)
						g_sAnkleAdj = 60;
		
					SetCurrentCommandList(&g_cmlsd1_Walk);		//command = 1
				}
				else if (iabs((short)g_ps2p.s.bRJoyLR-128) > TravelDeadZone) {
					JoyStickValueIn = (short)g_ps2p.s.bRJoyLR - 128;
					if ((JoyStickValueIn > TravelDeadZone) && (JoyStickValueIn < 70))
						SetCurrentCommandList((PCMLSD1)&s_cmlsd1_CCRight);		//command = 14
					else if (JoyStickValueIn >= 70)
						SetCurrentCommandList((PCMLSD1)&s_cmlsd1_TurnRight);		//command = 12
					else if ((JoyStickValueIn < -TravelDeadZone) && (JoyStickValueIn > -70))
						SetCurrentCommandList((PCMLSD1)&s_cmlsd1_CCLeft);		//command = 13
					else
						SetCurrentCommandList((PCMLSD1)&s_cmlsd1_TurnLeft);		//command = 11
				}
			
				// We will allow the other right UP/Down joystick change the hip angles to allow us to 
				// shoot.  Currenly only when we are not walking, but may change to instead of setting an independent
				// value we will instead build an offset from the movements.  But would probably need a button to press
				// to go back to zero...
				if (HServoIdle(LHPIN) && HServoIdle(RHPIN) && (iabs(((short)g_ps2p.s.bLJoyUD)-128) > TravelDeadZone) ) {
					#define MAX_HIP_DELTA 375
					sT = ((short)g_ps2p.s.bLJoyUD)-128;
					short int sLHipAngle = HServoPos(LHPIN);
					short int sRHipAngle = HServoPos(RHPIN);
#ifdef DEBUG
					if (g_fDebugBrat) 	{
						DBGPrintf("HM: %u %d %d %d ", g_ps2p.s.bLJoyUD, sT, sLHipAngle, sRHipAngle);
					}
#endif
					
					// First right hip
					sRHipAngle += sT/5;
					if (sRHipAngle > (1500 + MAX_HIP_DELTA))
						sRHipAngle = 1500 + MAX_HIP_DELTA;
					else if (sRHipAngle < (1500 - MAX_HIP_DELTA) )
						sRHipAngle = 1500 - MAX_HIP_DELTA;
						
					// Now Left hip	
					sLHipAngle -= sT/5;
					if (sLHipAngle > (1500 + MAX_HIP_DELTA))
						sLHipAngle = 1500 + MAX_HIP_DELTA;
					else if (sLHipAngle < (1500 - MAX_HIP_DELTA) )
						sLHipAngle = (1500 - MAX_HIP_DELTA);
#ifdef DEBUG
					if (g_fDebugBrat) 	{
						DBGPrintf("= %d %d\n\r", sLHipAngle, sRHipAngle);
					}
#endif
			
					HServoGroupMoveStart();
					HServo(RHPIN, sRHipAngle);
					HServo(LHPIN, sLHipAngle);
					HServoGroupMoveCommit(100);
				}		
			}
			//
			// some commands can be processed in the secondary loop...
			//

			// Check to see if we are asked for distance - test of bidirectional sending of data
			if (!g_ps2p.s.fNO && g_ps2pPrev.s.fNO) {			// O button test
#ifdef DEBUG
				// OK lets do a distance 
				u8 b = GetIRScanInCM(P18);		// get it into some dummy variable.
				
				// then tell remote to display the value for us.
				// We will use simple version where we pass the number(byte) to remote as the equiv of packet number
				// but later will add capabilities to pass strings as extra bytes...
				// setup to output back to our USB port

				DBGPrintf("Distance in cm: %d\n", b);
#endif
#ifdef USEXBEE
				XBeeOutputVal(b);
#endif				
			}

#ifdef DEBUG
			if (!g_ps2p.s.fNSquare && g_ps2pPrev.s.fNSquare) {	// Square button
				MSound(9, 1, 80, 2000);
				if (g_fDebugBrat)
					g_fDebugBrat = FALSE;		// turn off debug outputs
				else
					g_fDebugBrat = TRUE;		// Turn it back on.
			}
#endif
	
			// For R1 and L1 we want to fire guns? Do two transistion.  One when button down
			// second when the button comes back up.
	
			//
			// Handle the Turret with the Left joystick Left/right.  
			sT = ((short)g_ps2p.s.bLJoyLR)-128;
			if (iabs(sT) > TravelDeadZone)  {
				if (HServoIdle(TTPIN)) {
					// BUGBUG:: Hard code min and max for now
					u16 wT;
					u16 wOld = HServoPos(TTPIN);

					wT = wOld + sT;
					if (wT > 2250)
						wT = 2250;
					else if (wT < 750)
						wT = 750;
						 
#ifdef DEBUG
					if (g_fDebugBrat) 	{
						DBGPrintf("TM: %d %u %u\n\r", g_ps2p.s.bLJoyLR, wOld, wT);
					}
#endif
					// Last but not least do a group move of 1...
					if (wT != wOld) {
						HServoGroupMoveStart();
						HServo(TTPIN, wT);
						HServoGroupMoveCommit(100);
					}
					g_fCommandActive = TRUE;
				}
			}
		}
		// If we are not active and a timer has elapsed then go into low power mode...
		// go into the low power mode...
		if (g_fBotActive && !g_fCommandActive)	{		// only do this if we are actually still active and no command active...
			if (FTimeHasElapsed(g_ulLastActive, (unsigned long)10000))	{	//10 seconds
				SetCurrentCommandList((PCMLSD1)&s_cmlsd1_Rest);		//command = 9
				g_ulLastActive = GetCurrentTime();			// remember our time - so won't repeat for another 10 seconds...
			}
			else if (FTimeHasElapsed(g_ulLastActive, (unsigned long) 500))	//.5 seconds
				g_sAnkleAdj = 0;		// this will happen everytime through from .5 to 10 seconds, but will cost as much to avoid...
		}
	  
		CopyMem((char *)g_ps2pPrev.bData, (char *)g_ps2p.bData, sizeof(g_ps2p));
	}

	// Return to ok to give us a little break;
	return 20;
}


#ifdef USEXBEE
//==============================================================================
// This is code the checks for and processes input from the DIY XBee receiver
//			work
//==============================================================================
unsigned long CheckAndProcessDIYXBeeData(void)
{
	bool fPacketChanged;
	short	JoyStickValueIn;
	
	// Then try to receive a packet of information from the XBEE.
	ReceiveXBeePacket(&g_diyp);
	// See if we have a valid packet to process
	if (g_diystate.fPacketValid ) 
	{
		fPacketChanged = (FCompareMem((const char*)&g_diyp, (const char*)&g_diypPrev, sizeof(g_diyp)) != 0);
		if (fPacketChanged) 
		{
			if (g_diystate.fPacketForced)
				SendXbeeNewDataOnlyPacket(1);


#ifdef DEBUG_VERBOSE
			// setup to output back to our USB port
			if (g_fDebugBrat) {
				DBGPrintf("%x - %d %d - %d %d - %d %d\n", g_diyp.wButtons, g_diyp.bRJoyLR, g_diyp.bRJoyUD,
						g_diyp.bLJoyLR, g_diyp.bLJoyUD, g_diyp.bRSlider, g_diyp.bLSlider);
			}
#endif
		}


		// OK lets try "0" button for Start. 
		if ((g_diyp.wButtons & (1<<0)) && ((g_diypPrev.wButtons & (1<<0)) == 0)) {	//Start Button (0 on keypad) test
			if(g_fBotActive) {
				//Turn off
				//Sound P9,[100\4400,80\3800,60\3200];

      			MSound(9, 3, 100, 2200, 80, 1900, 60, 1600);
				SetCurrentCommandList(&s_cmlsd1_Rest);		//command = 9;
				g_fBotActive = FALSE;
			} else {
				//Turn on
				//Sound P9,[60\3200,80\3800,100\4400];
				XBeePlaySounds(s_abSND_RobotOn, 6);
      			MSound(9, 3, 60, 1600, 80, 1900, 100, 2200);
				SetCurrentCommandList(&s_cmlsd1_Home);		//command = 0
				g_fBotActive = TRUE;
			}
		}	
		
		if (g_fBotActive) {
			// We will use the Right Slider for this one...
			g_sTravLength = 8 - g_diyp.bRSlider/50; // should give us a range of about 3-8

			if ((g_diyp.wButtons & (1<<0xa)) && ((g_diypPrev.wButtons & (1<<0xa)) == 0))	// A button test
				SetCurrentCommandList(&s_cmlsd1_UpFromFront);		//command = 7

			else if ((g_diyp.wButtons & (1<<0xB)) && ((g_diypPrev.wButtons & (1<<0xB)) == 0))	// B button test
				SetCurrentCommandList(&s_cmlsd1_UpFromBack);		//command = 8
		
			else if ((g_diyp.wButtons & (1<<0x2)) && ((g_diypPrev.wButtons & (1<<0x2)) == 0))	// 2 button test
				SetCurrentCommandList(&s_cmlsd1_Home);		//command = 0
	
			// Commands that are set by joystick should only be set when a command is not currently active.	
			else if (!g_fCommandActive) {
				if (iabs(g_diyp.bRJoyUD-128) > TravelDeadZone) {
				 
					// Note, Speed and Angle here are in 10s so multiply by 10...		
					JoyStickValueIn = g_diyp.bRJoyUD - 128;
					if (JoyStickValueIn > 0) {
						g_lWalkSpeed1 = -(JoyStickValueIn*10)/g_sTravLength;
						g_lWalkAngle1 = (JoyStickValueIn + 40) - g_sAnkleAdj;
					} else {
						g_lWalkSpeed1 = -(JoyStickValueIn*10)/g_sTravLength;
						g_lWalkAngle1 = -((JoyStickValueIn - 40) + g_sAnkleAdj);
					}

					// We are setting up to do our main walk forward or backward.
					if (g_sLastStep == 0) {
						SET_MLSD1(&g_cmlsd1_Walk.amlsd1[0],g_lWalkAngle1, g_lWalkSpeed1, g_lWalkSpeed1, -g_lWalkAngle1,-g_lWalkSpeed1,-g_lWalkSpeed1,500-g_diyp.bLSlider);
						SET_MLSD1(&g_cmlsd1_Walk.amlsd1[1],-g_lWalkAngle1, g_lWalkSpeed1, g_lWalkSpeed1,  g_lWalkAngle1,-g_lWalkSpeed1,-g_lWalkSpeed1,500-g_diyp.bLSlider);
						g_sLastStep = 1;
					}
					else if (g_sLastStep == 1) {
						SET_MLSD1(&g_cmlsd1_Walk.amlsd1[0], -g_lWalkAngle1,-g_lWalkSpeed1,-g_lWalkSpeed1,  g_lWalkAngle1, g_lWalkSpeed1, g_lWalkSpeed1,500-g_diyp.bLSlider); 
						SET_MLSD1(&g_cmlsd1_Walk.amlsd1[1], g_lWalkAngle1,-g_lWalkSpeed1,-g_lWalkSpeed1, -g_lWalkAngle1, g_lWalkSpeed1, g_lWalkSpeed1,500-g_diyp.bLSlider); 
						g_sLastStep = 0;
					}
					g_sAnkleAdj = g_sAnkleAdj + 10;
					if (g_sAnkleAdj > 60)
						g_sAnkleAdj = 60;

		
					SetCurrentCommandList(&g_cmlsd1_Walk);		//command = 1
				}
				else if (iabs(g_diyp.bRJoyLR-128) > TravelDeadZone) {
					JoyStickValueIn = g_diyp.bRJoyLR - 128;
					if ((JoyStickValueIn > TravelDeadZone) && (JoyStickValueIn < 70))
						SetCurrentCommandList(&s_cmlsd1_CCRight);		//command = 14
					else if (JoyStickValueIn >= 70)
						SetCurrentCommandList(&s_cmlsd1_TurnRight);		//command = 12
					else if ((JoyStickValueIn < -TravelDeadZone) && (JoyStickValueIn > -70))
						SetCurrentCommandList(&s_cmlsd1_CCLeft);		//command = 13
					else
						SetCurrentCommandList(&s_cmlsd1_TurnLeft);		//command = 11
				}

				// We will allow the other right UP/Down joystick change the hip angles to allow us to 
				// shoot.  Currenly only when we are not walking, but may change to instead of setting an independent
				// value we will instead build an offset from the movements.  But would probably need a button to press
				// to go back to zero...
				if (HServoIdle(LHPIN) && HServoIdle(RHPIN) && (iabs((short int)g_ps2p.s.bLJoyUD-128) > TravelDeadZone) ) {
					#define MAX_HIP_DELTA 375
					short int sLHipAngle = HServoPos(LHPIN);
					short int sRHipAngle = HServoPos(RHPIN);
					
					// First right hip
					sRHipAngle += (((short int)g_ps2p.s.bLJoyUD -128)/5);
					if (sRHipAngle > (1500 + MAX_HIP_DELTA))
						sRHipAngle = 1500 + MAX_HIP_DELTA;
					else if (sRHipAngle < (1500 - MAX_HIP_DELTA) )
						sRHipAngle = 1500 - MAX_HIP_DELTA;
						
					// Now Left hip	
					sLHipAngle -= (((short int)g_ps2p.s.bLJoyUD -128)/5);
					if (sLHipAngle > (1500 + MAX_HIP_DELTA))
						sLHipAngle = 1500 + MAX_HIP_DELTA;
					else if (sLHipAngle < (1500 - MAX_HIP_DELTA) )
						sLHipAngle = (1500 - MAX_HIP_DELTA);
			
					HServoGroupMoveStart();
					HServo(RHPIN, sRHipAngle);
					HServo(LHPIN, sLHipAngle);
					HServoGroupMoveCommit(100);
				}		
			
			}
			//
			// some commands can be processed in the secondary loop...
			//

			// Check to see if we are asked for distance - test of bidirectional sending of data
			if ((g_diyp.wButtons & (1<<0xD)) && ((g_diypPrev.wButtons & (1<<0xD)) == 0)) {	// D button test 
				// OK lets do a distance 
				u8 b = GetIRScanInCM(P18);		// get it into some dummy variable.
				
				// then tell remote to display the value for us.
				// We will use simple version where we pass the number(byte) to remote as the equiv of packet number
				// but later will add capabilities to pass strings as extra bytes...
#ifdef DEBUG
				DBGPrintf("Distance in cm: %d\n", b);
#endif
				XBeeOutputVal(b);
			}

#ifdef DEBUG
			if ((g_diyp.wButtons & (1<<0xe)) && ((g_diypPrev.wButtons & (1<<0xe)) == 0)) {	// E button test
				if (g_fDebugBrat) {
					XBeeOutputString("Debug Off");
					g_fDebugBrat = FALSE;		// turn off debug outputs
				} else {
					g_fDebugBrat = TRUE;		// Turn it back on.
					XBeeOutputString("Debug On");
				}
			}
#endif
	
			// For now on DIY will use keypad 0 and F but later may add special buttons...
			// For R1 and L1 we want to fire guns? Do two transistion.  One when button down
			// second when the button comes back up.
#if 0
			if ((g_diyp.wButtons & (1<<0xf)) != (g_diypPrev.wButtons & (1<<0xf))) {	// f button test
				if (g_diyp.wButtons & (1<<0xf)) { // F button test
					// Button Down - If wating for cycle to complete quickly end it, in case our internal switch fails to work...
					if (RGunWaitingToCycle)
						hservo (RGUNP\-7000\0);
					else
						hservo (RGUNP\7000\0);
				} else {
					// Button up - for now two different quick sounds
					if (!RGunWaitingToCycle)
						RGunWaitingToCycle = TRUE;				// Button back up, will wait until gun fires to turn off
					else
						RGunWaitingToCycle = FALSE;				// Don't need to wait any longer
				}
			
			} 
			
			
			if ((g_diyp.wButtons & (1<<0x0)) != (g_diypPrev.wButtons & (1<<0x0))) {	// 0 button test
				if (g_diyp.wButtons & (1<<0x0)) {
					// Button Down
					if (LGunWaitingToCycle)
						hservo (LGUNP\-7000\0); 					// again failsafe if the gun internal switch fails
					else
						hservo (LGUNP\7000\0);
				} else {
					if (!LGunWaitingToCycle)
						LGunWaitingToCycle = TRUE;				// Button back up, will wait until gun fires to turn off
					else
						LGunWaitingToCycle = FALSE;				// don't need to wait for it to end any more
				}
			} 
	
#endif
			if (iabs(g_diyp.bLJoyLR-128) > TravelDeadZone) 	{
				if (HServoIdle(TTPIN)) {
					// BUGBUG:: Hard code min and max for now
					u16 wT;
					u16 wOld = HServoPos(TTPIN);

					wT = wOld + ((short)(g_diyp.bLJoyLR-128))*5;
					if (wT > 2250)
						wT = 2250;
					else if (wT < 750)
						wT = 750;
						 
					// Last but not least do a group move of 1...
					if (wT != wOld) {
						HServoGroupMoveStart();
						HServo(TTPIN, wT);
						HServoGroupMoveCommit(200);
					}
					g_fCommandActive = TRUE;
				}
			}
		}
		// If we are not active and a timer has elapsed then go into low power mode...
		// go into the low power mode...
		if (g_fBotActive && !g_fCommandActive) {			// only do this if we are actually still active and no command active...
			if (FTimeHasElapsed(g_ulLastActive, (unsigned long)10000)) {	//10 seconds
				SetCurrentCommandList(&s_cmlsd1_Rest);		//command = 9
				g_ulLastActive = GetCurrentTime();			// remember our time - so won't repeat for another 10 seconds...
			}
			else if (FTimeHasElapsed(g_ulLastActive, (unsigned long) 500))	//.5 seconds
				g_sAnkleAdj = 0;		// this will happen everytime through from .5 to 10 seconds, but will cost as much to avoid...
		}
	  
		g_diypPrev = g_diyp;
	}

	else if (g_fBotActive && g_diystate.fPacketForced) 	{
		// Ok are active and we lost communications with the remote.  So turn off!!!
		// no use sending the message to play sounds, but may try anyway 
		g_fBotActive = FALSE;
		XBeePlaySounds(s_abSND_RobotOff, 6);
		SetCurrentCommandList(&s_cmlsd1_Rest);		//command = 9;
	}


	// Return to caller saying we don't need any delays...
	return 0;
}
#endif //USEXBEE


//==============================================================================
//[GetIRSensorInCM] - Get the distance to a target using a Sharp IR sensor.  These
// devices are not linear so we use a look up table to convert the results to CM.
// Parameters: IRPIN - Which pin to use
// Return: Distance in CM.  Range is from 0 to 80
//==============================================================================
// Define the table of values to convert
// This table is used to convert Sharp IR readings into CM
static const u8 s_abIRCMTable[] = {80,80,80,80,80,80,80,80,80,78,
                        76,74,72,70,68,66,64,62,60,59,
                        58,57,55,53,52,51,50,49,48,47,
                        45,43,42,41,40,39,38,37,35,33, 
                        32,31,30,30,29,29,28,28,27,27, 
                        26,26,26,25,25,25,24,24,24,23, 
                        23,22,22,21,21,20,20,20,19,19, 
                        18,18,18,17,17,16,16,16,15,15, 
                        15,14,14,13,13,13,12,12,11,11, 
                        11,10,10,10,10,10,10,10,10,10 };



short GetIRScanInCM(u8 bPin)
{                        
  	// first read in the IRvalue.
	u16 wScanRange = adin(bPin);

  	// If it is greater than 512 it is out of range we will return 0
	if (wScanRange > 512)		// outside our table so return 0 to signal too close 
		return 0;

	// now convert entry to cm from table.  Old program
	// divided value by 5.12 using floating point, but could simply multiply by 100 and divide by 512
	// in integer math, which should be a lot quicker.  512*100 < 65536 so fits in word...
	return (short) s_abIRCMTable[(wScanRange*100)/512];
}

//==============================================================================
// SetCurrentCommandList; simple helper function that sets the current command list
//==============================================================================
void SetCurrentCommandList(const PCMLSD1 pcmlsd1)
{
#ifdef DEBUG
	// setup to output back to our USB port
	if (g_fDebugBrat) {
		unsigned long tc =  GetCurrentTime();
		DBGPrintf("\nSetCurrentCommandList(%lu): %x %x\n", tc - g_ulLastDebug,(unsigned short)pcmlsd1 & 0xffff, FLegServosStillMoving());
		g_ulLastDebug = tc;
	}
#endif
	g_pcmlsd1 = pcmlsd1;
	g_imlsd1 = -1;					// setup count of the item list to -1 F Process will handle it...
	g_fCommandActive = TRUE;		// make sure we don't time out...
}


//==============================================================================
// FProcessCurrentCommandList - Main function that does a query to see if the 
// 		multiple servo code needs to update.  If a command finishes it then sees if
//		there are more commands to process if so starts up the next one...
//==============================================================================
bool FProcessCurrentCommandList(void)
{
	if (!g_pcmlsd1)
		return FALSE;		// no current command

	// Then see if our leg servos are still active
	if (FLegServosStillMoving())
		return TRUE;		// at least one of our leg servos is still moving...


	// Or see if we have some delay defined
	if (g_ulPausePeriod) {
		if (!FTimeHasElapsed(g_ulPauseStart, g_ulPausePeriod))
			return TRUE;		// nope - say a command is still active
		g_ulPausePeriod = 0;		// clear it out.
	}

#ifdef DEBUG
	if (g_fDebugBrat) {
		DBGPrintf("FPCCL Next\n\r");
	}
#endif


	g_imlsd1++;				// inrement to the next command in our list (note started at minus one so should be fine
	if (g_imlsd1 < g_pcmlsd1->cItems) {
		// we need to convert each of these values to drive speed
		PMLSD1 plmsd1 = &(g_pcmlsd1->amlsd1[g_imlsd1]);
#ifdef DEBUG
		if (g_fDebugBrat) {
			// setup to output back to our USB port
			unsigned long tc =  GetCurrentTime();
			DBGPrintf("M(%lu) %x: RAKH:%d %d %d, L: %d %d %d t:%d\n\r", (long)(tc - g_ulLastDebug),  (long)FLegServosStillMoving(), 
				(long)plmsd1->rightanklepos, (long)plmsd1->rightkneepos,(long)plmsd1->righthippos,
				plmsd1->leftanklepos,plmsd1->leftkneepos,plmsd1->lefthippos,plmsd1->sMoveTime);
			g_ulLastDebug = tc;
		}
#endif
		// Setup a special case.  If the move time is less than zero than this is a pause for the positive amount
		// of time, not an actual group move item.
		if (plmsd1->sMoveTime < 0 ) {
			g_ulPauseStart = GetCurrentTime();					// remember when we started this delay
			g_ulPausePeriod = ((long)-(plmsd1->sMoveTime));	
		} else {
			g_ulPausePeriod = 0;	// clear the pause period so we won't use it anymore...
#ifdef DEBUG
			if (g_fDebugBrat) 	{
				DBGPrintf("FPCL: %d %d %d : %d %d %d", HServoPos(RHPIN), HServoPos(RKPIN), HServoPos(RAPIN),
							HServoPos(LHPIN), HServoPos(LKPIN), HServoPos(LAPIN));
				DBGPrintf(" => %d %d %d : %d %d %d\n\r",  
						ANGLE1_TO_SERVO_PW(plmsd1->righthippos), ANGLE1_TO_SERVO_PW(plmsd1->rightkneepos), ANGLE1_TO_SERVO_PW(plmsd1->rightanklepos),
						ANGLE1_TO_SERVO_PW(-plmsd1->lefthippos), ANGLE1_TO_SERVO_PW(-plmsd1->leftkneepos), ANGLE1_TO_SERVO_PW(-plmsd1->leftanklepos));
			}
#endif			
		
			HServoGroupMoveStart();						// start a group move
			HServo(RAPIN, ANGLE1_TO_SERVO_PW(plmsd1->rightanklepos));
			HServo(RKPIN, ANGLE1_TO_SERVO_PW(plmsd1->rightkneepos));
			HServo(RHPIN, ANGLE1_TO_SERVO_PW(plmsd1->righthippos));
			HServo(LAPIN, ANGLE1_TO_SERVO_PW(-plmsd1->leftanklepos));
			HServo(LKPIN, ANGLE1_TO_SERVO_PW(-plmsd1->leftkneepos));
			HServo(LHPIN, ANGLE1_TO_SERVO_PW(-plmsd1->lefthippos));
			HServoGroupMoveCommit(plmsd1->sMoveTime);			// Do the commit, pass in how long you wish for the move to take
		}
		return TRUE;
	}

#ifdef DEBUG
		if (g_fDebugBrat) {
			DBGPrintf("FPCCL Done\n\r");
		}
#endif

	g_pcmlsd1 = 0;		// Null out the list
	return FALSE;		// we processed our whole command list
}		

//==============================================================================
// FLegServosStillMoving: Simple helper function to see if any of the leg
//				servos are still moving toward their new position.
//==============================================================================
bool FLegServosStillMoving(void)
{
	register bool fRet;
#ifdef DEBUG_ENTERLEAVE
	if (g_fDebugBrat) {
		// setup to output back to our USB port
		DBGPrintf("E FLSSM\n\r");
	}
#endif

	if (HServoIdle(RAPIN) && HServoIdle(RKPIN) && HServoIdle(RHPIN) &&
			HServoIdle(LAPIN) && HServoIdle(LKPIN) && HServoIdle(LHPIN))
		fRet = FALSE;
	else 
		fRet = TRUE;
	
#ifdef DEBUG_ENTERLEAVE
	if (g_fDebugBrat) {
		// setup to output back to our USB port
		DBGPrintf("X FLSSM %x\n\r", fRet);
	}
#endif
	return fRet;
}

//==============================================================================
// MoveLegServosDegrees1: simple helper function to convert the six servos from degrees to DRIVE_SPEED units and then call off to my
// 				groupmove function.
//==============================================================================
void MoveLegServosDegrees1 (short rightanklepos,short rightkneepos,short righthippos,
		short leftanklepos,short leftkneepos,short lefthippos,short sMoveTime)
{
	// we need to convert each of these values to drive speed
#ifdef DEBUG
	if (g_fDebugBrat) {
		// setup to output back to our USB port
		unsigned long tc =  GetCurrentTime();
		DBGPrintf("M: RAKH(%lu) %x:", (long)(tc - g_ulLastDebug), FLegServosStillMoving());
		DBGPrintf("%d %d %d", rightanklepos,rightkneepos,righthippos);
		DBGPrintf(" L: %d %d %d t:%d\n", leftanklepos,leftkneepos,lefthippos,sMoveTime);
		g_ulLastDebug = tc;
	}
#endif

	HServoGroupMoveStart();						// start a group move
	HServo(RAPIN, ANGLE1_TO_SERVO_PW(rightanklepos));
	HServo(RKPIN, ANGLE1_TO_SERVO_PW(rightkneepos));
	HServo(RHPIN, ANGLE1_TO_SERVO_PW(righthippos));
	HServo(LAPIN, ANGLE1_TO_SERVO_PW(leftanklepos));
	HServo(LKPIN, ANGLE1_TO_SERVO_PW(leftkneepos));
	HServo(LHPIN, ANGLE1_TO_SERVO_PW(lefthippos));
//	DBGPrintf("Before HServo Commit\n\r");
	HServoGroupMoveCommit(sMoveTime);			// Do the commit, pass in how long you wish for the move to take
//	DBGPrintf("After HServo Commit\n\r");

}

//=============================================================================

#ifdef PS2_TEST
void PS2_Test(void)
{
	bool fFailReported;
	u8 bDupCount;
	// OK lets move all of the servos to their zero point.

	DBGPrintf("PS2 Controller Test\r");

	fFailReported = false;
	bDupCount = 20;
	
	for(;;) {
		// See what is happening to PS2 lines?  Is HSIO kicking in?
		if (IO.PMR1.BIT.TXD || SCI3.SCR3.BIT.RE || SCI3.SCR3.BIT.TE) {
			disable();
			DBGPrintf("HSIO bits enabled??? %x %x %x\r",IO.PMR1.BIT.TXD ? 1:0,  SCI3.SCR3.BIT.RE ? 1 : 0, SCI3.SCR3.BIT.TE ? 1 : 0);
			MSound(9, 3, 60, 1600, 80, 1900, 100, 2200);
			enable();
			pause(2000);
		}

		if (PS2_Query(g_ps2p.bData, sizeof(g_ps2p))) {
			if ((FCompareMem((const char *)g_ps2p.bData, (const char *)g_ps2pPrev.bData, sizeof(g_ps2p)) != 0) || (bDupCount > 10)) {
				g_ps2pPrev = g_ps2p;
//				disable ();	 		//enable timer interrupt
	

				DBGPrintf("%2x%2x\r",g_ps2p.bData[0],g_ps2p.bData[1] );
				//DBGPrintf("%2x%2x %2x %2x %2x %2x\r",g_ps2p.bData[0],g_ps2p.bData[1], g_ps2p.s.bRJoyLR, g_ps2p.s.bRJoyUD, g_ps2p.s.bLJoyLR, g_ps2p.s.bLJoyUD);
//				enable ();	 		//enable timer interrupt

				bDupCount = 0;
			}
			fFailReported = false;
			bDupCount++;
		} else if (! fFailReported)	{
			DBGPrintf("PS2_Query failed\r");
			fFailReported = true;
		}
			
		//get user entered data
#if 0		
		if (TASerin(_S_IN_, 0, I9600, (char*)&data, 1, 50000, (word)-1)  && (data >= 10) && (data <= 15))
			break;		
#endif			
		pause(50);
	}

	DBGPrintf("PS2 Test exit\n");
}
#endif


//=============================================================================
#ifdef SERVO_TEST
void ServoRangeTest(void)
{

	int data;
	bool fExit = FALSE;	// when to exit
	u16 center_us;		// The number of microseconds pulse length to center the servo. On ideal servos it will be 1500. ie 1,5ms
	u16 range_us;		// The number of microseconds either side of center for full range. On ideal servos it be 500. Range = 1ms to 2ms.



	// OK lets move the turret servo to 0
	DBGPrintf("Servo Range Test - See where center and end points are\n");


	// Wait until button is released...
	while (button_pressed())
		;
	range_us = svoTurret.range_us;
	center_us = svoTurret.center_us;
	DBGPrintf("Center: %d Range: %u\n", center_us, range_us);

	while(!fExit) {
		// if button is pressed again, abort from here.  This gives us a way to handle if I had the button pressed
		// but no serial port...
		if (button_pressed())
			return;

		//get user entered data
		data = GET_DATA_USB;
		//if data received
		if (data !=-1) {
			if ((data >= 10) && (data <=15))
				fExit = TRUE;	// not sure how the keypad will map so give NL, CR, LF... all implies exit

			else if ((data == '+') || (data == '-')) {
				if (data == '+')
					range_us += 25;		
				else
					range_us -= 25;

				DBGPrintf("Range: %u\n", range_us);
				servoSetConfig(&svoTurret, center_us, range_us);  // update the center point and then go there...
				act_setSpeed(&svoTurret,  0);			
			}		

			else if (data == '4')
				act_setSpeed(&svoTurret,  -127);
			else if (data == '5')
				act_setSpeed(&svoTurret,  0);
			else if (data == '6')
				act_setSpeed(&svoTurret,  127);
			else if (data == '7')
				act_setSpeed(&svoTurret,  -64);
			else if (data == '9')
				act_setSpeed(&svoTurret,  64);
		}

	}

}
#endif

//==============================================================================
// Subroutine: ReadServoOffsets
// Will read in the zero points that wer last saved for the different servos
// that are part of this robot.  
//
//==============================================================================
void ReadServoOffsets(void)
{
	u8	abHdr[2];		// first byte is cnt of servos, 2nd is checksum
	u8	bCSCalc;
	u8	*p;
	u8	i;

	short asServoOffsets[NUMSERVOS];			// This is up in phoenix_v32.bas..
	
	// First read in the header data	
	DBGPrintf("ReadServoOffsets");
	Read(0, 2, abHdr);
	DBGPrintf(" %d %x\n\r", abHdr[0], abHdr[1]);
	

	if ((abHdr[0] > 0) && (abHdr[0] <= NUMSERVOS)) {
	
		// Assume my read will fit only 7 servos...	
		Read(2, abHdr[0]*2, (u8*)asServoOffsets);
		
		DBGPrintf("RSO - Read EEPROM\n\r");
		//... calculate checksum...
		bCSCalc = 0;
		p = (u8*)asServoOffsets;
		
		for (i=0; i < abHdr[0]; i++) {
			DBGPrintf("%d(%x) ", asServoOffsets[i],  asServoOffsets[i]);
			bCSCalc += *p++;
			bCSCalc += *p++;
		}	
	
		DBGPrintf("RSO - CS %x\n\r", bCSCalc);
		if (bCSCalc == abHdr[1]) {
			// Warning: The servooffsets are compatible with the Basic version which is in the Basic HServo
			// units which are in clock ticks.  So we need to convert back to ms units...
			for (i = 0; i < abHdr[0]; i++)
				HServoSetServoOffset(c_abServoTbl[i],asServoOffsets[i]/(F_CPU/1000000)); 
		}
		else {
			DBGPrintf("Checksum error in Servo Offsets\n\r");
		}
		DBGPrintf("\n\r");
	}
		
}
