//==============================================================================
// GLOBALS - The main global definitions for the CPhenix program - still needs
//		to be cleaned up.
// This program assumes that the main files were compiled as C files
//==============================================================================
#ifndef _HEX_GLOBALS_H_
#define _HEX_GLOBALS_H_

#define ARC32_PHOENIX

#define BASICATOMPROARC32
//#define BASICATOMPRO40	//try a 40
//#define BASICATOMPRO28
#ifndef BASICATOMPRO28
#ifndef BASICATOMPRO40
#ifndef BASICATOMPROARC32
#warning "Which Atom Pro not defined assuming BASICATOMPRO28"
#define BASICATOMPRO28
#endif
#endif
#endif

#ifdef ARC32_PHOENIX
#include "Phoenix_cfg.h"
#else
#include "chr3_Cfg.h"
#endif

#include "..\baplib\baplib.h"
#include "..\baplib\eeprom.h"

#ifndef USE_SSC32
#include "..\baplib\servo.h"
#endif

#define _GLOBALS_H_

#ifdef BASICATOMPRO28
#define DBGSerial 	ssDBG
#define SSCSerial	ssSSC		
#define XBeeSerial	Serial
extern SoftwareSerial	ssDBG;
extern SoftwareSerial	ssSSC;
extern HardwareSerial	Serial;
#else
#define DBGSerial 	Serial
#define SSCSerial	ssSSC		
#define XBeeSerial	Serial1
extern SoftwareSerial	ssSSC;
extern HardwareSerial	Serial;
extern HardwareSerial	Serial1;
#endif

//=============================================================================
//[CONSTANTS]
//=============================================================================
#define BUTTON_DOWN 0
#define BUTTON_UP 	1

#define	c1DEC		10
#define	c2DEC		100
#define	c4DEC		10000
#define	c6DEC		1000000

#define	cRR			0
#define	cRM			1
#define	cRF			2
#define	cLR			3
#define	cLM			4
#define	cLF			5

extern void GaitSelect(void);
extern short SmoothControl (short CtrlMoveInp, short CtrlMoveOut, byte CtrlDivider);


//-----------------------------------------------------------------------------
// Define Global variables
//-----------------------------------------------------------------------------
extern boolean			g_fHexOn;				//Switch to turn on Phoenix
extern boolean			g_fPrev_HexOn;			//Previous loop state 

//Body position
extern long		BodyPosX; 		//Global Input for the position of the body
extern long		BodyPosY; 
extern long		BodyPosZ; 

//Body Inverse Kinematics
extern long		BodyRotX1; 		//Global Input pitch of the body
extern long		BodyRotY1;		//Global Input rotation of the body
extern long		BodyRotZ1; 		//Global Input roll of the body


//[gait]
extern byte			GaitType;			//Gait type
extern word		NomGaitSpeed;		//Nominal speed of the gait

extern short		LegLiftHeight;		//Current Travel height
extern long			TravelLengthX;		//Current Travel length X
extern long			TravelLengthZ; 		//Current Travel length Z
extern long			TravelRotationY; 	//Current Travel Rotation Y

//[Single Leg Control]
extern byte			SelectedLeg;
extern short		SLLegX;
extern short		SLLegY;
extern short		SLLegZ;
extern boolean			fSLHold;		 	//Single leg control mode

//--------------------------------------------------------------------
//[GP PLAYER]
// define the offset in the ARC32 EEPROM where we emulate the SSC-32 from
#define ARC32_SSC_OFFSET	0x400		// we will offset saving all of the SSC-32 by this amount

extern boolean	GPStart;						//Start the GP Player
extern byte	GPSeq;							//Number of the sequence
extern boolean	GPEnable;						//Enables the GP player when the SSC version ends with "GP<cr>"

//[Balance]
extern boolean BalanceMode;

//[TIMING]
extern byte			InputTimeDelay;	//Delay that depends on the input to get the "sneaking" effect
extern word			SpeedControl;	//Adjustible Delay
extern word			SSCTime;		//Time for servo updates

// maybe should do this different
extern void	ServoDriverStart(void);
extern void	ServoDriverCommit(void);
extern void FreeServos(void);


extern void MSound(byte _pin, byte cNotes, ...);
extern int DBGPrintf(const char *format, ...);
extern int SSCPrintf(const char *format, ...);
extern int SSCRead (byte* pb, int cb, word wTimeout, word wEOL);

#ifndef BASICATOMPRO28
extern void TerminalMonitorInit(void);
extern void TerminalMonitor (u8);
#endif



// The defined controller must provide the following
extern void InitController(void);
extern void	ControlInput(void);
extern void	AllowControllerInterrupts(boolean fAllow);


// debug handler...

// debug handler...
#define DBG_LVL_NORMAL			0x01	// - Normal starting debug level
#define DBG_LVL_VERBOSE	 		0x02	// - More verbose debug
#define DBG_LVL_CONTROL			0x04	// - Turn on debug outputs for the control (PS2,XBEE...)
#define DBG_LVL_ENTERLEAVE 		0x80	// Enter/Leave

extern bool g_fDBGHandleError;
extern u16 	g_wDebugLevel;


#endif


